/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-DPM-12                **/
/**                                                                       **/
/**                             Version 1.C.                              **/
/**                                                                       **/
/**  This file contains routines for Borland C++/3.1 to allow the user,   **/
/**  to write programs for the MODULAR-4 in "C"                           **/
/**                                                                       **/
/**             Project start    : 24.05.1996                             **/
/**             Last change      : 10.10.1997 (Rev. 1.C.)                 **/
/**                                                                       **/
/**                         Author : H.D. & A.H.                          **/
/**                         Status : tested                               **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/** This library was tested with Borland C++ Version 3.1.                 **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1996, 97                       **/
/**                                                                       **/
/***************************************************************************/
/***************************************************************************/
/* Revision History:                                                       */
/* 1.A to 1.B timeout function added                                       */
/* 1.B to 1.C 'open' in m044_download_file checked for NULL instead of -1  */
/*            long cons added                                              */
/***************************************************************************/


/* The switches for the library are set in the configuration files of Pascal
   (*.TP file) und C (project-file).

   They can also be set in this source file. Then the following #defines
   have to exist:

   If you use Pascal, #define BORLAND_PASCAL has to be included
   (default is C).

   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8 and #define ML8RT have to be included
   (default is PC library).

   If you work with Windows, this is detected by the compiler.
   No #defines have to be included.
*/

#ifdef ML8RT
/* As the parameters micro_slot and offset have changed in the function
   ml8rt_read_eeprom, using ml8rtbib version 1.C. and less the instruction
   #define ML8RT_VERSION_1C has to be added
*/
#endif


#include "..\..\..\c\m044_lib.h"


#ifdef BORLAND_PASCAL
   #define STATIC static                /* for definition of arrays */
#else
   #define STATIC static far            /* for definition of arrays */
#endif

#define ANZ_MOD 11                     /* MODULAR-4/486: number of modul
					  slots: 10+1 (slot 0 is not used) */


#ifndef ML8RT
   #include "ML8BIB.H"
   #include <stdio.h>
   #include <io.h>
   #include <fcntl.h>
				       /* the functions of library ML8BIB
					  all begin with ml8_... : */
   #define mlx_read_eeprom ml8_read_eeprom_copy
   #define mlx_out8_port ml8_out8_port
   #define mlx_out16_port ml8_out16_port
   #define mlx_in8_port ml8_in8_port
   #define mlx_in16_port ml8_in16_port
#else
   #include "stdio.h"
   #include "dos.h"
   #include "ML8RTBIB.H"

   #define mlx_out8_port(port, dat1) outportb(port, dat1)
   #define mlx_out16_port(port, dat1) outport(port, dat1)
   #define mlx_in8_port(port) inportb(port)
   #define mlx_in16_port(port) inport(port)
#endif


#ifdef ML8RT
   #ifdef ML8RT_VERSION_1C
      #define mlx_read_eeprom(micro_slot, offset) ml8rt_read_eeprom_copy(offset, micro_slot)
   #else
      #define mlx_read_eeprom(micro_slot, offset) ml8rt_read_eeprom_copy(micro_slot, offset)
   #endif
#endif


const STATIC ushort MBA[ANZ_MOD]
   = {0x000,
      0x400, 0x500, 0x600, 0x700,
      0x420, 0x440, 0x460, 0x480, 0x4A0, 0x4C0};


#ifdef __DLL__
					/* if module DLL shall be
					   generated */
#include <windows.h>
#pragma argsused

HANDLE   hInst;

int far pascal LibMain(hInstance, wDataSeg, wHeapSize, lpszCmdLine)
   HANDLE  hInstance;                   /* instance of DLL */
   WORD    wDataSeg;                    /* data segment */
   WORD    wHeapSize;                   /* size of heap */
   LPSTR   lpszCmdLine;                 /* pointer to command line */

   {
   hInst = hInstance;
   if(wHeapSize > 0) UnlockData(0);
   return(1);
   }

#endif  /* __DLL__ */

#ifndef BORLAND_PASCAL
#endif

#include <stdlib.h>
#include <dos.h>





STATIC m044_slv_datastr_type SlaveDataStrucBuf [ANZ_MOD][124];
STATIC ushort                ErrorStruct [ANZ_MOD][20];



/****************************************************************************
							     m044_bib_startup
****************************************************************************/
ushort EXPORT m044_bib_startup(void)
{
 return(0);
}


/****************************************************************************
							       m044_configure
****************************************************************************/
ushort EXPORT m044_configure(byte micro_slot)
{
 byte slave, i;

 ErrorStruct[micro_slot][0] = CONFIGURE;

 if((mlx_read_eeprom(micro_slot, 0x00) & 0x00FF) == 0x002C)
   {
    for(i = 0; i < 20; i++)
      ErrorStruct[micro_slot][i] = 0;

    for(slave = 0; slave < 124; slave++)
      {
       SlaveDataStrucBuf[micro_slot][slave].inp_ptr      = 0;
       SlaveDataStrucBuf[micro_slot][slave].outp_ptr     = 0;
       SlaveDataStrucBuf[micro_slot][slave].diag_ptr     = 0;
       SlaveDataStrucBuf[micro_slot][slave].diag_len_ptr = 0;
       SlaveDataStrucBuf[micro_slot][slave].diag_cnt_ptr = 0;
       SlaveDataStrucBuf[micro_slot][slave].inp_len      = 0;
       SlaveDataStrucBuf[micro_slot][slave].outp_len     = 0;
       SlaveDataStrucBuf[micro_slot][slave].inout        = 0;
       SlaveDataStrucBuf[micro_slot][slave].slave_type   = 0;
      }

    return(0);
   }
 else return(0x1be0);                  /* no M-DPM-12 in slot 'micro_slot' */
}


/****************************************************************************
							 m044_get_master_mode
****************************************************************************/
ushort EXPORT m044_get_master_mode(byte micro_slot, byte *mode)
{
 ErrorStruct[micro_slot][0] = GET_MASTER_MODE;
					/* set DPRAM pointer to status cell
					   and return read byte */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_STATUS);
 *mode = (byte)mlx_in16_port(M044_RW16_DPR_BYTE);
 return(0);
}



/*  ###################################################################### */
/*                  P R O F I B U S    P A R A M E T E R S                 */
/*  ###################################################################### */

/****************************************************************************
								m044_download
****************************************************************************/
ushort EXPORT m044_download
	(byte micro_slot, void *data, byte data_size, byte flag)
{
 unsigned long timeout = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = DOWNLOAD;
				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength=data_size             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 mlx_out16_port(M044_RW16_DPR_BYTE, ((ushort)data_size));
				       /* set DPRAM pointer to area data and
					  entry datas                      */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 m044_set_dpram_block(micro_slot, data, data_size);
 switch(flag)
   {
    case _M044_NO_MORE_DATA :          /* last block:
					  set DPRAM pointer to cell command
					  and set command 0x08, semaphore
					  <> 0                             */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0108);
       break;

    case _M044_MORE_DATA :
				       /* following blocks:
					  set DPRAM pointer to cell command
					  and set command 0x88, semaphore
					  <> 0                             */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0188);
       break;
   }
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

 while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));

 if(timeout > 0)
   {
    mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				       /* set DPRAM pointer to data area
					  and read returned datas          */
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);

    ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);
   }
 else ErrorStruct[micro_slot][1] = 2;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
								  m044_upload
****************************************************************************/
ushort EXPORT m044_upload
	(byte micro_slot, void *data, byte *data_size, byte *flag)
{
 unsigned long timeout = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = UPLOAD;

 switch(*flag)
   {
    case _M044_START_DATA :
				      /* first block:
					 set DPRAM pointer to cell command
					 and set command 0x09, semaphore
					 <> 0 */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0109);
       break;

    case _M044_MORE_DATA :
				      /* following block:
					 set DPRAM pointer to cell command
					 and set command 0x89, semaphore
					 <> 0                              */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0189);
       break;
  }

				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

 while(((ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD)) < 0x0100) && (--timeout));

				       /* set DPRAM pointer to cell
					  datalength and read datalength   */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 *data_size = (byte)mlx_in16_port(M044_RW16_DPR_BYTE);
				       /* set DPRAM pointer to area data and
					  read datas                       */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 m044_get_dpram_block(micro_slot, data, *data_size);
				       /* set DPRAM pointer to reply cell
					  and reset reply                  */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
 ErrorStruct[micro_slot][1] &= 0x00FF;
 switch(ErrorStruct[micro_slot][1])
   {
    case 0x09 :
       *flag = _M044_NO_MORE_DATA;
       ErrorStruct[micro_slot][1] = 0;
       break;
    case 0x89 :
       *flag = _M044_MORE_DATA;
       ErrorStruct[micro_slot][1] = 0;
       break;
   }

 if(timeout == 0) ErrorStruct[micro_slot][1] = 2;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
						       m044_set_slave_address
****************************************************************************/
ushort EXPORT m044_set_slave_address
	(byte micro_slot, void *data, byte size)
{
 unsigned long timeout = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = SET_SLAVE_ADDRESS;
				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = size                */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 mlx_out16_port(M044_RW16_DPR_BYTE, ((ushort)size));
				       /* set DPRAM pointer to area data and
					  entry datas                      */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 m044_set_dpram_block(micro_slot, data, size);
				       /* set DPRAM pointer to cell command
					  and set command 0x11, semaphore
					  <> 0                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0111);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);
 while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				       /* set DPRAM pointer to data area
					  and read returned datas          */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);

 if(timeout == 0) ErrorStruct[micro_slot][1] = 6;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
								 m044_restart
****************************************************************************/
ushort EXPORT m044_restart(byte micro_slot, byte restart_par)
{
 unsigned long timeout = 0xffffffffL;
 ushort reply;

 ErrorStruct[micro_slot][0] = RESTART;
				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = 1                   */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 mlx_out16_port(M044_RW16_DPR_BYTE, 1);
				       /* set DPRAM pointer to area data and
					  entry reset_byte                 */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 mlx_out16_port(M044_RW16_DPR_BYTE, ((ushort)restart_par));
				       /* set DPRAM pointer to cell command
					  and entry command=0x01, semaphore
					  <> 0                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0101);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 do{
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);
    reply = mlx_in16_port(M044_RW16_DPR_WORD);
   }while((reply < 0x0100) && (--timeout));

 if(timeout > 0)
   {
    mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				       /* set DPRAM pointer to data area
					  and read returned word           */
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
    ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);
   }
 else
   ErrorStruct[micro_slot][1] = 1;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
						m044_refresh_slave_datastruct
****************************************************************************/
ushort EXPORT m044_refresh_slave_datastruct(byte micro_slot)
{
 byte *data_ptr;
 byte size;
 ushort reply;
 unsigned long timeout1 = 0x00ffffffL, timeout2;

 ErrorStruct[micro_slot][0] = REFRESH_SLAVE_DATASTRUCT;

 data_ptr = (byte *)&SlaveDataStrucBuf[micro_slot][0];
 reply = 0;

 while(((reply & 0x00FF) != 0x0003) && (--timeout1))
   {
				       /* set DPRAM pointer to cell command
					  and set command:
					  if reply  = 0 : 0x03
					  if reply <> 0 : 0x83             */
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
    if(!reply)
      mlx_out16_port(M044_RW16_DPR_WORD, 0x0103);
    else
      mlx_out16_port(M044_RW16_DPR_WORD, 0x0183);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0                             */
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);
    timeout2 = 0x00ffffffL;
    while(((reply = mlx_in16_port(M044_RW16_DPR_WORD)) < 0x0100) && (--timeout2));
    if(timeout2 == 0) break;
				       /* copy delivered datas to data
					  structure buffer                 */
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
    size = (byte)mlx_in16_port(M044_RW16_DPR_BYTE);
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
    m044_get_dpram_block(micro_slot, data_ptr, size);
    data_ptr = data_ptr + size;
				       /* reset reply                      */
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);
    mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
   }

 if((timeout1 == 0) || (timeout2 == 0)) ErrorStruct[micro_slot][1] = 1;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 return(0);
}



/*  ###################################################################### */
/*                         M A S T E R    C O N T R O L                    */
/*  ###################################################################### */

/****************************************************************************
							 m044_set_master_mode
****************************************************************************/
ushort EXPORT m044_set_master_mode(byte micro_slot, byte command)
{
 unsigned long timeout = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = SET_MASTER_MODE;
				       /* set DPRAM pointer to cell
					  command                          */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
				       /* evaluate parameter command       */
 switch(command)
   {
    case _M044_SYNCHRONIZE :
				       /* set command 0x02, semaphore <> 0 */
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0102);
       break;
    case _M044_STOP :
				       /* set command 0x04, semaphore <> 0 */
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0104);
       break;
    case _M044_CLEAR :
				       /* set command 0x05, semaphore <> 0 */
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0105);
       break;
    case _M044_OPERATE :
				       /* set command 0x06, semaphore <> 0 */
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0106);
       break;
   }
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

 while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));

 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				      /* set DPRAM pointer to data area
					 and read returned datas           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);

 if(timeout == 0) ErrorStruct[micro_slot][1] = 1;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
							 m044_watchdog_enable
****************************************************************************/
ushort EXPORT m044_watchdog_enable(byte micro_slot, ushort timeout)
{
 unsigned long timecnt = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = WATCHDOG_ENABLE;
				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = 2                   */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 mlx_out16_port(M044_RW16_DPR_BYTE, 2);
				       /* set DPRAM pointer to area data and
					  entry timeout                    */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 mlx_out16_port(M044_RW16_DPR_WORD, timeout);
				       /* set DPRAM pointer to cell command
					  and set command 0x07, semaphore
					  <> 0                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0107);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

 while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timecnt));

 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				       /* set DPRAM pointer to data area
					  and read returned datas          */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);

 if(timecnt == 0) ErrorStruct[micro_slot][1] = 1;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);

}


/****************************************************************************
						      m044_watchdog_retrigger
****************************************************************************/
ushort EXPORT m044_watchdog_retrigger(byte micro_slot)
{
 ErrorStruct[micro_slot][0] = WATCHDOG_RETRIGGER;
				       /* set DPRAM pointer to watchdog    */
				       /* cell and write 0x00 to retrigger */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_WATCHDOG);
 mlx_out16_port(M044_RW16_DPR_BYTE, 0x0000);

 return(0);
}



/*  ###################################################################### */
/*                         S L A V E    A C C E S S                        */
/*  ###################################################################### */

/****************************************************************************
							  m044_set_slave_byte
****************************************************************************/
ushort EXPORT m044_set_slave_byte
	(byte micro_slot, byte slave, byte subchannel, byte data, byte cons)
{
 cons = cons;
 ErrorStruct[micro_slot][0] = SET_SLAVE_BYTE;
				       /* check parameter subchannel;
					  if error ErrorStruct =  100      */
 if(subchannel >= SlaveDataStrucBuf[micro_slot][slave].outp_len)
   {
    ErrorStruct[micro_slot][1] = 100;
    return(0x40e0);
   }
				       /* set byte                         */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].outp_ptr + (ushort)subchannel));
				       /* byte = always consistence        */
 m044_set_dpram_byte(micro_slot, data, _M044_FIXED_PTR);

 return(0);
}


/****************************************************************************
							  m044_set_slave_word
****************************************************************************/
ushort EXPORT m044_set_slave_word
	(byte micro_slot, byte slave, byte subchannel, ushort data, byte cons)
{
 ErrorStruct[micro_slot][0] = SET_SLAVE_WORD;
				       /* check parameter subchannel;
					  if error, set unpossible cons    */
 if(subchannel >= (2 * SlaveDataStrucBuf[micro_slot][slave].outp_len))
   cons = _M044_CONS + _M044_NO_CONS;
				       /* set DPRAM pointer                */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].outp_ptr + 2 * ((ushort)subchannel)));
 switch(cons)
   {
    case _M044_CONS :                  /* word = always consistence        */
       m044_set_dpram_word(micro_slot, data, _M044_FIXED_PTR);
       break;

    case _M044_NO_CONS :	       /* word = always consistence        */
       m044_set_dpram_word(micro_slot, data, _M044_FIXED_PTR);
       break;

    case _M044_CONS+_M044_NO_CONS:
       ErrorStruct[micro_slot][1] = 100;
       break;
   }

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
							 m044_set_slave_block
****************************************************************************/
ushort EXPORT m044_set_slave_block
	(byte micro_slot, byte slave, ushort offset, byte size, void *data, byte cons)
{
 unsigned long timeout = 0x0ffff;
 ErrorStruct[micro_slot][0] = SET_SLAVE_BLOCK;

				       /* check parameter subchannel;
					  if error, set unpossible cons    */
 if(size > SlaveDataStrucBuf[micro_slot][slave].outp_len)
   {
    ErrorStruct[micro_slot][1] = 102;  /* size > data error                */
    return(0x40e0);
   }

 cons = (SlaveDataStrucBuf[micro_slot][slave].inout & 0x0f) >> 1;

 if((cons & 0x04) || (cons & 0x02))    /* buffered or long consistency     */
 {
				       /* set DPRAM pointer                */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS,
	 (SlaveDataStrucBuf[micro_slot][slave].outp_ptr) + offset);

       m044_set_dpram_block(micro_slot, data, size);

				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = 2                   */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
       mlx_out16_port(M044_RW16_DPR_BYTE, 1);
				       /* set DPRAM pointer to area data and
					  entry slave address              */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
       mlx_out16_port(M044_RW16_DPR_WORD, slave);
				       /* set DPRAM pointer to cell command
					  and set command 0x13, semaphore
					  <> 0                             */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0113);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

       while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));

       mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);

				       /* set DPRAM pointer to data area
					  and read returned datas          */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
       ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);

       if(timeout == 0) ErrorStruct[micro_slot][1] = 1;

       if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);

       return(0);
 }

				       /* set DPRAM pointer                */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
   (SlaveDataStrucBuf[micro_slot][slave].outp_ptr) + offset);

 if(cons & 0x01)                       /* short consistency                */
   {				       /* read block with consistence      */
       mlx_out8_port(M044_W8_CONSISTENCE, 2);
       while((mlx_in8_port(M044_R8_STATUS) & 0x20) != 0);
   }
 m044_set_dpram_block(micro_slot, data, size);

 if(cons & 0x01)                       /* short consistency                */
   {				       /* check, if consistency error      */
       if(mlx_in8_port(M044_R8_CONSISTENCE) & 0x01)
	  ErrorStruct[micro_slot][1] = 101;
       else
	  ErrorStruct[micro_slot][1] = 0;
   }

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}




/****************************************************************************
							  m044_get_slave_byte
****************************************************************************/
ushort EXPORT m044_get_slave_byte
	(byte micro_slot, byte slave, byte subchannel, byte *data, byte cons)
{
 cons = cons;
 ErrorStruct[micro_slot][0] = GET_SLAVE_BYTE;
				       /* check parameter subchannel;
					  if error return 100              */
 if(subchannel >= SlaveDataStrucBuf[micro_slot][slave].inp_len)
   {
    ErrorStruct[micro_slot][1] = 100;
    return(0x40e0);
   }
				      /* read byte */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].inp_ptr + (ushort)subchannel));
				      /* byte = always consistence         */
 *data = m044_get_dpram_byte(micro_slot, _M044_FIXED_PTR);

 return(0);
}


/****************************************************************************
							  m044_get_slave_word
****************************************************************************/
ushort EXPORT m044_get_slave_word
	(byte micro_slot, byte slave, byte subchannel, ushort *data,
	byte cons)
{
 ErrorStruct[micro_slot][0] = GET_SLAVE_WORD;
				       /* check parameter subchannel;
					  if error, set unpossible cons    */
 if(subchannel >= (2 * SlaveDataStrucBuf[micro_slot][slave].inp_len))
   cons = _M044_CONS + _M044_NO_CONS;
				       /* set DPRAM pointer                */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].inp_ptr + 2 * ((ushort)subchannel)));
 switch (cons)
   {
    case _M044_CONS :		       /* word = always consistence        */
       *data = m044_get_dpram_word(micro_slot, _M044_FIXED_PTR);
       break;

    case _M044_NO_CONS :	       /* word = always consistence        */
       *data = m044_get_dpram_word(micro_slot, _M044_FIXED_PTR);
       break;

    case _M044_CONS + _M044_NO_CONS:
       ErrorStruct[micro_slot][1] = 100;
       break;
   }

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
// 11                                                    m044_get_slave_block
****************************************************************************/
ushort EXPORT m044_get_slave_block
	(byte micro_slot, byte slave, ushort offset, byte size, void *data, byte cons)
{
 unsigned long timeout = 0x0ffff;

 ErrorStruct[micro_slot][0] = GET_SLAVE_BLOCK;
				       /* check parameter subchannel;
					  if error, set unpossible cons    */
 if(size > SlaveDataStrucBuf[micro_slot][slave].inp_len)
   {
    ErrorStruct[micro_slot][1] = 102;  /* size > data error                */
    return(0x40e0);
   }

 cons = (SlaveDataStrucBuf[micro_slot][slave].inout & 0xf0) >> 5;

 if((cons & 0x04) || (cons & 0x02))    /* buffered or long consistency     */
 { 				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = 2                   */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
       mlx_out16_port(M044_RW16_DPR_BYTE, 1);
				       /* set DPRAM pointer to area data and
					  entry slave address              */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
       mlx_out16_port(M044_RW16_DPR_WORD, slave);
				       /* set DPRAM pointer to cell command
					  and set command 0x14, semaphore
					  <> 0                             */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
       mlx_out16_port(M044_RW16_DPR_WORD, 0x0114);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

       while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));

       mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);

				       /* set DPRAM pointer to data area
					  and read returned datas          */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
       ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);

       if(timeout == 0) ErrorStruct[micro_slot][1] = 1;

       if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);

				       /* set DPRAM pointer                */
       mlx_out16_port(M044_RW16_DPR_POINTER_POS,
	 (SlaveDataStrucBuf[micro_slot][slave].inp_ptr) + offset);

       m044_get_dpram_block(micro_slot, data, size);
       return(0);
 }

				       /* set DPRAM pointer                */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
   (SlaveDataStrucBuf[micro_slot][slave].inp_ptr) + offset);

 if(cons & 0x01)                       /* short consistency                */
   {				       /* read block with consistence      */
       mlx_out8_port(M044_W8_CONSISTENCE, 1);
       while((mlx_in8_port(M044_R8_STATUS) & 0x20) != 0);
   }
 m044_get_dpram_block(micro_slot, data, size);

 if(cons & 0x01)                       /* short consistency                */
   {				       /* check, if consistency error      */
       if(mlx_in8_port(M044_R8_CONSISTENCE) & 0x01)
	  ErrorStruct[micro_slot][1] = 101;
       else
	  ErrorStruct[micro_slot][1] = 0;
   }

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}


/****************************************************************************
						       m044_set_slave_command
****************************************************************************/
ushort EXPORT m044_set_slave_command
	(byte micro_slot, byte command, byte group, byte slave)
{
 unsigned long timeout = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = SET_SLAVE_COMMAND;
				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = 3                   */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 mlx_out16_port(M044_RW16_DPR_BYTE, 3);
				       /* set DPRAM pointer to area data and
					  entry parameters command, group
					  and slave                        */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 mlx_out16_port(M044_RW16_DPR_BYTE_INC, ((ushort)command));
 mlx_out16_port(M044_RW16_DPR_BYTE_INC, (((ushort)group) << 8));
 mlx_out16_port(M044_RW16_DPR_BYTE, ((ushort)slave));
				       /* set DPRAM pointer to cell command
					  and set command 0x10, semaphore
					  <> 0                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0110);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

 while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));

 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				       /* set DPRAM pointer to data area
					  and read returned datas          */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 ErrorStruct[micro_slot][1]  = mlx_in16_port(M044_RW16_DPR_WORD);

 if(timeout == 0) ErrorStruct[micro_slot][1] = 1;

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 else return(0);
}



/*  ###################################################################### */
/*                   P R O F I B U S    D I A G N O S I S                  */
/*  ###################################################################### */

/****************************************************************************
						    m044_get_slave_datastruct
****************************************************************************/
ushort EXPORT m044_get_slave_datastruct
	(byte micro_slot, byte slave, m044_slv_datastr_type *data)
{
 byte *ptr8_1, *ptr8_2;
 int i;

 ErrorStruct[micro_slot][0] = GET_SLAVE_DATASTRUCT;

 ptr8_1 = (byte *)&SlaveDataStrucBuf[micro_slot][slave];
 ptr8_2 = (byte *)data;

 if((micro_slot <= 10) && (slave < 124))
   {
    for(i = 0; i < sizeof(SlaveDataStrucBuf[micro_slot][slave]); i++)
      {
       *ptr8_2++ = *ptr8_1++;
      }
    // *data = SlaveDataStrucBuf[micro_slot][slave]; (C only !)
    return(0);
   }
 else
   {
    ErrorStruct[micro_slot][1] = 0x0001;
    return(0x40e0);
   }
}


/****************************************************************************
						m044_get_slave_diagnosis_list
****************************************************************************/
ushort EXPORT m044_get_slave_diagnosis_list
	(byte micro_slot, m044_slv_bitmap_type *data)
{
 ErrorStruct[micro_slot][0] = GET_SLAVE_DIAGNOSIS_LIST;
				       /* set DPRAM pointer to slave
					  diagnosis list                   */
 if(micro_slot <= 10)
   {
    mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_SLAVE_DIAG_LIST);
    m044_get_dpram_block(micro_slot, data, sizeof(m044_slv_bitmap_type));
    return(0);
   }
 else
   {
    ErrorStruct[micro_slot][1] = 0x0001;
    return(0x40e0);
   }
}


/****************************************************************************
						   m044_check_slave_diagnosis
****************************************************************************/
ushort EXPORT m044_check_slave_diagnosis(byte micro_slot, byte slave, byte *diaglen)
{
 div_t flag_par;

 ErrorStruct[micro_slot][0] = CHECK_SLAVE_DIAGNOSIS;
				       /* calculate flag parameters        */
 flag_par.quot = slave / 16;
 flag_par.rem  = slave % 16;
				       /* set DPRAM pointer to slave
					  diagnosis list [field]           */
 mlx_out16_port
    (M044_RW16_DPR_POINTER_POS,
     (M044_DPR_SLAVE_DIAG_LIST + 2 * ((ushort)flag_par.quot)));
 if(mlx_in16_port(M044_RW16_DPR_WORD) & (1 << (ushort)flag_par.rem))
   {
    mlx_out16_port(M044_RW16_DPR_POINTER_POS,
       (SlaveDataStrucBuf[micro_slot][slave].diag_len_ptr));
    *diaglen = m044_get_dpram_byte(micro_slot, _M044_FIXED_PTR);
    return(0);
   }
 else
   {
    *diaglen = 0;
    return(0);
   }
}



/****************************************************************************
						     m044_get_slave_diagnosis
****************************************************************************/
ushort EXPORT m044_get_slave_diagnosis
	(byte micro_slot, byte slave, byte size, void *data)
{
 byte diag_cnt_in;
 byte diag_cnt_out;

 ErrorStruct[micro_slot][0] = GET_SLAVE_DIAGNOSIS;
				       /* check parameters, if error in
					  parameter, return error code
					  100                              */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].diag_len_ptr));
 if(m044_get_dpram_byte(micro_slot, _M044_FIXED_PTR) < size)
   {
    ErrorStruct[micro_slot][1] = 100;
    return(0x40e0);
   }
				       /* read diagnosis counter in        */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].diag_cnt_ptr));
 diag_cnt_in = m044_get_dpram_byte(micro_slot, _M044_FIXED_PTR);
				       /* copy diagnosis datas             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].diag_ptr));
 m044_get_dpram_block(micro_slot, data, size);
				       /* read diagnosis counter out       */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS,
    (SlaveDataStrucBuf[micro_slot][slave].diag_cnt_ptr));
 diag_cnt_out = m044_get_dpram_byte(micro_slot, _M044_FIXED_PTR);
				       /* consistence conflict ?
					 If conflict, return error code
					 101                               */
 if(diag_cnt_in != diag_cnt_out)
   {
    ErrorStruct[micro_slot][1] = 101;
    return(0x40e0);
   }
 return(0);
}


/****************************************************************************
						  m044_get_data_transfer_list
****************************************************************************/
ushort EXPORT m044_get_data_transfer_list
	(byte micro_slot, m044_slv_bitmap_type *data)
{
 ErrorStruct[micro_slot][0] = GET_DATA_TRANSFER_LIST;
				       /* set DPRAM pointer to data transfer
					  list                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_TRANSFER_LIST);
 m044_get_dpram_block(micro_slot, data, sizeof(m044_slv_bitmap_type));
 return(0);
}


/****************************************************************************
						      m044_check_slave_active
****************************************************************************/
ushort EXPORT m044_check_slave_active(byte micro_slot, byte slave, byte *active)
{
 div_t flag_par;

 ErrorStruct[micro_slot][0] = CHECK_SLAVE_ACTIVE;
				       /* calculate flag parameters        */
 flag_par.quot = slave / 16;
 flag_par.rem  = slave % 16;
				       /* set DPRAM pointer to data transfer
					  list [field]                     */
 mlx_out16_port
    (M044_RW16_DPR_POINTER_POS,
     (M044_DPR_DATA_TRANSFER_LIST + 2 * ((ushort)flag_par.quot)));
 if(mlx_in16_port(M044_RW16_DPR_WORD) & (1 << (ushort)flag_par.rem))
   *active = 1;
 else
   *active = 0;
 return(0);
}


/****************************************************************************
						m044_get_master_status_struct
****************************************************************************/
ushort EXPORT m044_get_master_status_struct
	(byte micro_slot, m044_mst_statusstr_type *data)
{
 ErrorStruct[micro_slot][0] = GET_MASTER_STATUS_STRUCT;
					/* set DPRAM pointer to master
					   status channel and read channel */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_MASTER_STATUS);
 m044_get_dpram_block(micro_slot, data, sizeof(m044_mst_statusstr_type));
 return(0);
}


/****************************************************************************
						 m044_get_system_error_struct
****************************************************************************/
ushort EXPORT m044_get_system_error_struct
	(byte micro_slot, m044_mst_syserr_type *data)
{
 ErrorStruct[micro_slot][0] = GET_SYSTEM_ERROR_STRUCT;
				       /* set DPRAM pointer to system error
					  channel and read channel         */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_SYSTEM_ERROR);
 m044_get_dpram_block(micro_slot, data, sizeof(m044_mst_syserr_type));
 return(0);
}


/****************************************************************************
						       m044_poll_system_error
****************************************************************************/
ushort EXPORT m044_poll_system_error(byte micro_slot, ushort *syserr)
{
 ErrorStruct[micro_slot][0] = POLL_SYSTEM_ERROR;
				       /* set DPRAM pointer to system
					  error, read first word and
					  return read word                 */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_SYSTEM_ERROR);
 *syserr = mlx_in16_port(M044_RW16_DPR_WORD);

 return(0);
}



/*  ###################################################################### */
/*                     D I R E C T    D P R A M    A C C E S S             */
/*  ###################################################################### */

/****************************************************************************
						       m044_set_dpram_pointer
****************************************************************************/
void EXPORT m044_set_dpram_pointer(byte micro_slot, ushort position)
{
				       /* set DPRAM pointer to position    */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, position);
 return;
}


/****************************************************************************
						       m044_get_dpram_pointer
****************************************************************************/
ushort EXPORT m044_get_dpram_pointer(byte micro_slot)
{
				       /* return DPRAM pointer position    */
 return(mlx_in16_port(M044_RW16_DPR_POINTER_POS));
}


/****************************************************************************
							  m044_set_dpram_byte
****************************************************************************/
void EXPORT m044_set_dpram_byte(byte micro_slot, byte data, byte ptr_mode)
{
				       /* check even DPRAM pointer         */
 if(mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01)
   {                                   /* pointer is odd                   */
    if(ptr_mode == _M044_FIXED_PTR)
      mlx_out16_port(M044_RW16_DPR_BYTE, (((ushort)data) << 8));
    else
      mlx_out16_port(M044_RW16_DPR_BYTE_INC, (((ushort)data) << 8));
    }
 else
   {                                   /* pointer is even                  */
    if(ptr_mode == _M044_FIXED_PTR)
      mlx_out16_port(M044_RW16_DPR_BYTE, ((ushort)data));
    else
      mlx_out16_port(M044_RW16_DPR_BYTE_INC, ((ushort)data));
    }
 return;
}


/****************************************************************************
							  m044_set_dpram_word
****************************************************************************/
void EXPORT m044_set_dpram_word(byte micro_slot, ushort data, byte ptr_mode)
{
 ushort dpram_ptr_pos;
				       /* check even DPRAM pointer         */
 if((dpram_ptr_pos = mlx_in16_port(M044_RW16_DPR_POINTER_POS)) & 0x01)
   {                                   /* pointer is odd                   */
    ushort data_copy;
    data_copy = data;
    if(ptr_mode == _M044_FIXED_PTR)
      {                                /* set low byte to odd address,
					  set high byte to even address,
					  restore changed dpram pointer    */
       mlx_out16_port(M044_RW16_DPR_BYTE_INC, (((ushort)data_copy) << 8));
       mlx_out16_port(M044_RW16_DPR_BYTE, (((ushort)data) >> 8));
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, dpram_ptr_pos);
      }
    else
      {                                /* set low byte to odd address,
					  set high byte to even address    */
       mlx_out16_port(M044_RW16_DPR_BYTE_INC, (((ushort)data_copy) << 8));
       mlx_out16_port(M044_RW16_DPR_BYTE_INC, (((ushort)data) >> 8));
      }
   }
 else
   {                                   /* pointer is even                  */
    if(ptr_mode == _M044_FIXED_PTR)
       mlx_out16_port(M044_RW16_DPR_WORD, data);
    else
       mlx_out16_port(M044_RW16_DPR_WORD_INC, data);
   }
 return;
}


/****************************************************************************
							 m044_set_dpram_block
****************************************************************************/
void EXPORT m044_set_dpram_block(byte micro_slot, void *data, byte size)
{
 ushort *ptr16;
 byte *ptr8;

 ptr16 = (ushort *)data;
 ptr8  = (byte *)data;
				       /* if DPRAM pointer odd, then write
					  first byte                       */
 if((mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01) && (size > 0))
   {
    mlx_out16_port
       (M044_RW16_DPR_BYTE_INC, (((ushort)*ptr16) << 8));
    size--;
    ptr8++;
    ptr16 = (ushort *)ptr8; // = (ushort *)(((byte *)data)++);
   }
				       /* write words with auto-increment  */
 while(size > 1)
   {
    mlx_out16_port(M044_RW16_DPR_WORD_INC, *ptr16);
    ptr16++;
    size -= 2;
   }
				       /* if rest<>0, write last byte      */
 if(size)
   mlx_out16_port(M044_RW16_DPR_BYTE_INC, *ptr16);
 return;
}


/****************************************************************************
							  m044_get_dpram_byte
****************************************************************************/
byte EXPORT m044_get_dpram_byte(byte micro_slot, byte ptr_mode)
{
 ushort data;
				       /* check even DPRAM pointer         */
 if(mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01)
   {                                   /* pointer is odd                   */
    if(ptr_mode == _M044_FIXED_PTR)
       data = mlx_in16_port(M044_RW16_DPR_BYTE) >> 8;
    else
       data = mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8;
   }
 else
   {                                   /* pointer is even                  */
    if(ptr_mode == _M044_FIXED_PTR)
       data = mlx_in16_port(M044_RW16_DPR_BYTE);
    else
       data = mlx_in16_port(M044_RW16_DPR_BYTE_INC);
   }
 return((byte)data);
}


/****************************************************************************
							  m044_get_dpram_word
****************************************************************************/
ushort EXPORT m044_get_dpram_word(byte micro_slot, byte ptr_mode)
{
 ushort data, dpram_ptr_pos;
				       /* check even DPRAM pointer         */
 if((dpram_ptr_pos = mlx_in16_port(M044_RW16_DPR_POINTER_POS)) & 0x01)
   {                                   /* pointer is odd                   */
    ushort data_low;
    if(ptr_mode == _M044_FIXED_PTR)
      {                                /* get low byte from odd position,
					  get high byte from even address,
					  restore changed dpram pointer    */
       data_low = mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8;
       data = (mlx_in16_port(M044_RW16_DPR_BYTE) << 8) | data_low;
       mlx_out16_port(M044_RW16_DPR_POINTER_POS, dpram_ptr_pos);
      }
    else
      {                                /* get low byte from odd position,
					  get high byte from even address  */
       data_low = mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8;
       data = (mlx_in16_port(M044_RW16_DPR_BYTE_INC) << 8) | data_low;
      }
   }
 else
   {                                   /* pointer is even                  */
    if(ptr_mode == _M044_FIXED_PTR)
      data = mlx_in16_port(M044_RW16_DPR_WORD);
    else
      data = mlx_in16_port(M044_RW16_DPR_WORD_INC);
   }
 return(data);
}


/****************************************************************************
							 m044_get_dpram_block
****************************************************************************/
void EXPORT m044_get_dpram_block(byte micro_slot, void *data, byte size)
{
 ushort *ptr16;
 byte *ptr8;
				       /* if DPRAM pointer odd, then read
					  first byte                       */
 if((mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01) && (size > 0))
   {
    ptr8 = (byte *)data;
    *ptr8 = (byte)(mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8);
    size--;
    ptr8++;
    data = (void *)ptr8;
   }
				       /* read words with auto-increment   */
 ptr16 = (ushort *)data;
 while(size > 1)
   {
    *ptr16 = mlx_in16_port(M044_RW16_DPR_WORD_INC);
    ptr16++;
    size -= 2;
   }
				       /* if rest <> 0, read last byte     */
 if(size)
   {
    ptr8 = (byte *)ptr16;
    *ptr8 = (byte)mlx_in16_port(M044_RW16_DPR_BYTE_INC);
   }
 return;
}


/*  ###################################################################### */
/*                     D I R E C T    F P G A    A C C E S S               */
/*  ###################################################################### */


/****************************************************************************
							      m044_hard_reset
****************************************************************************/
ushort EXPORT m044_hard_reset(byte micro_slot)
{
 int i;

 mlx_out8_port(M044_W8_CONSISTENCE, 0x00);
 mlx_out8_port(M044_W8_RESET, 1);
 mlx_out8_port(M044_W8_IO_PINS, 0x8F);
				       /* CLEAR STATUS CELLS IN DPRAM     */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, 0x3EFC);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, 0x3ED0);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);

 for(i = 0; i < 200; i++)	       /* XRESET active for 44 s         */
   mlx_out8_port(M044_W8_IO_PINS, 0x8F);

 mlx_out8_port(M044_W8_IO_PINS, 0x0F); /* deactivate XRESET 		  */
 for(i = 0; i < 20; i++)
   ErrorStruct[micro_slot][i] = 0;

 return(0);
}

/****************************************************************************
							      m044_hard_reset
****************************************************************************/
ushort EXPORT m044_reset_master(byte micro_slot)
{
 int i;

 mlx_out8_port(M044_W8_CONSISTENCE, 0x00);
 mlx_out8_port(M044_W8_IO_PINS, 0x8F);
				       /* CLEAR STATUS CELLS IN DPRAM     */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, 0x3EFC);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, 0x3ED0);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);

 for(i = 0; i < 200; i++)	       /* XRESET active for 44 s         */
   mlx_out8_port(M044_W8_IO_PINS, 0x8F);

 mlx_out8_port(M044_W8_IO_PINS, 0x0F); /* deactivate XRESET 		  */

 for(i = 0; i < 20; i++)
   ErrorStruct[micro_slot][i] = 0;

 return(0);
}


/****************************************************************************
							m044_get_fpga_version
****************************************************************************/
byte EXPORT m044_get_fpga_version(byte micro_slot)
{
 return(mlx_in8_port(M044_R8_GATE_ARRAY_VERSION));
}


/****************************************************************************
						      m044_set_modul_register
****************************************************************************/
void EXPORT m044_set_modul_register
	(byte micro_slot, byte reg_type, byte data)
{
 switch(reg_type)
   {
    case _M044_ISR :
       mlx_out8_port(M044_RW8_INT_SEL_REGISTER, data);
       break;
    case _M044_SMR :
       mlx_out8_port(M044_RW8_SER_MODE_SEL_REGISTER, data);
       break;
    case _M044_LER :
       mlx_out8_port(M044_RW8_LED_REGISTER, data);
       break;
   }
 return;
}


/****************************************************************************
						      m044_get_modul_register
****************************************************************************/
byte EXPORT m044_get_modul_register(byte micro_slot, byte reg_type)
{
 byte data;

 switch (reg_type)
   {
    case _M044_ISR :
       data = mlx_in8_port(M044_RW8_INT_SEL_REGISTER);
       break;
    case _M044_IST :
       data = mlx_in8_port(M044_R8_INT_STATUS_REGISTER);
       break;
    case _M044_SMR :
       data = mlx_in8_port(M044_RW8_SER_MODE_SEL_REGISTER);
       break;
    case _M044_LER :
       data = mlx_in8_port(M044_RW8_LED_REGISTER);
       break;
   }
 return(data);
}


/****************************************************************************
							m044_configure_io_pin
****************************************************************************/
void EXPORT m044_configure_io_pin(byte micro_slot, ushort data)
{
 mlx_out16_port(M044_RW16_IO_PIN_MODE_REGISTER, data);
 return;
}


/****************************************************************************
						       m044_get_io_pin_config
****************************************************************************/
ushort EXPORT m044_get_io_pin_config(byte micro_slot)
{
 return(mlx_in16_port(M044_RW16_IO_PIN_MODE_REGISTER));
}


/****************************************************************************
							      m044_set_io_pin
****************************************************************************/
void EXPORT m044_set_io_pin(byte micro_slot, ushort data)
{
 mlx_out16_port(M044_RW16_IO_PINS, data);
 return;
}


/****************************************************************************
							      m044_get_io_pin
****************************************************************************/
ushort EXPORT m044_get_io_pin(byte micro_slot)
{
 return(mlx_in16_port(M044_RW16_IO_PINS));
}


/****************************************************************************
						     m044_set_timeout_counter
****************************************************************************/
void EXPORT m044_set_timeout_counter(byte micro_slot, ushort data)
{
 mlx_out16_port(M044_RW16_TIMEOUT_COUNTER, data);
 return;
}


/****************************************************************************
						     m044_get_timeout_counter
****************************************************************************/
ushort EXPORT m044_get_timeout_counter(byte micro_slot)
{
 return(mlx_in16_port(M044_RW16_TIMEOUT_COUNTER));
}


/****************************************************************************
								m044_set_cons
****************************************************************************/
void EXPORT m044_set_cons(byte micro_slot, byte mode)
{
 mlx_out8_port(M044_W8_CONSISTENCE, mode);
 return;
}


/****************************************************************************
							      m044_clear_cons
****************************************************************************/
byte EXPORT m044_clear_cons(byte micro_slot)
{
 return(mlx_in8_port(M044_R8_CONSISTENCE));
}


/****************************************************************************
							m044_get_modul_status
****************************************************************************/
byte EXPORT m044_get_modul_status(byte micro_slot)
{
 return(mlx_in8_port(M044_R8_STATUS));
}



/*  ###################################################################### */
/*                  B I N A R Y    F I L E    T R A N S F E R              */
/*  ###################################################################### */

#ifndef ML8RT

/****************************************************************************
							   m044_download_file
****************************************************************************/
ushort EXPORT m044_download_file
	(byte micro_slot, const char *binary_file, ushort *downloaded)
{
 ushort error;

#ifndef BORLAND_PASCAL
 ushort fsize, i;
 int handle, size;
 byte data[250];

 ErrorStruct[micro_slot][0] = DOWNLOAD_FILE;

				       /* open returns -1 if error         */
 if((handle = open(binary_file, O_RDONLY | O_BINARY)) == -1) return(0x40e0);

 fsize = (ushort)filelength(handle);
 *downloaded = 0;
 for(i = 0; i < ((fsize / 200) + 1); i++)
   {
    size = read(handle, data, 200);
    if(size == 200)
      {
       error = m044_download(micro_slot,
			     data,
			     size,
			     _M044_MORE_DATA);

       if(error != 0) return(error);
      }
    else
      {
       error = m044_download(micro_slot,
			     data,
			     size,
			     _M044_NO_MORE_DATA);

       if(error != 0) return(error);
      }
    *downloaded += size;
   }
 close(handle);
#else                                  /* BORLAND_PASCAL                   */
 ErrorStruct[micro_slot][0] = DOWNLOAD_FILE;
 error = m044_download_file_p(micro_slot, binary_file, downloaded);
#endif
 return(error);
}


/****************************************************************************
							     m044_upload_file
****************************************************************************/
ushort EXPORT m044_upload_file
	(byte micro_slot, const char *binary_file, ushort *uploaded)
{
 ushort error = 0;

#ifndef BORLAND_PASCAL
 byte binary[255];
 byte flag, size;
 FILE *f;
 ushort len;

 ErrorStruct[micro_slot][0] = UPLOAD_FILE;
 *uploaded = 0;

 if((f = fopen(binary_file, "wb")) != NULL)
   {
    flag = _M044_START_DATA;
    do
      {
       error = m044_upload(micro_slot, &binary, &size, &flag);
       if(error) return(error);
       len = (byte) fwrite(&binary, 1, size, f);
       *uploaded = *uploaded + len;

       if(len != size)
	 {
	  ErrorStruct[micro_slot][1] = 1;
	  error = 0x40e0;
	 }
      }
    while((flag == _M044_MORE_DATA) && (!error));
   }
 else
   {
    ErrorStruct[micro_slot][1] = 1;
    error = 0x40e0;
   }
 fclose(f);

#else                                  /* BORLAND_PASCAL                   */
 error = m044_upload_file_p(micro_slot, binary_file, uploaded);
#endif

 return(error);
}
#endif // IFNDEF ML8RT

/****************************************************************************
							       m044_get_error
****************************************************************************/
ushort EXPORT m044_get_error(byte micro_slot, ushort *data)
{
 int i;

 for(i = 0; i < 10; i++)
   {
    data[i] = ErrorStruct[micro_slot][i];
    ErrorStruct[micro_slot][i] = 0;
   }

 return(0);                            /* no M-DPM-12 in slot 'micro_slot' */
}


/****************************************************************************
																					m044_get_error
****************************************************************************/
void EXPORT m044_set_tasknum(ushort task)
{
 task = task;
}


/****************************************************************************
								m044_fw_timer
****************************************************************************/
ushort EXPORT m044_fw_timer (byte micro_slot, ushort timer)
{
 unsigned long timeout = 0x00ffffffL;

 ErrorStruct[micro_slot][0] = FW_TIMER;
				       /* set DPRAM pointer to cell
					  datalength and entry
					  datalength = 2                   */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATA_LENGTH);
 mlx_out16_port(M044_RW16_DPR_BYTE, 2);
				       /* set DPRAM pointer to area data and
					  entry reset_byte                 */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 mlx_out16_port(M044_RW16_DPR_WORD, timer);
				       /* set DPRAM pointer to cell command
					  and entry command=0x12, semaphore
					  <> 0                             */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_COMMAND);
 mlx_out16_port(M044_RW16_DPR_WORD, 0x0112);
				       /* set DPRAM pointer to cell reply
					  and wait until reply semaphore
					  <> 0, then reset reply           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_REPLY);

 while((mlx_in16_port(M044_RW16_DPR_WORD) < 0x0100) && (--timeout));

 mlx_out16_port(M044_RW16_DPR_WORD, 0x0000);
				       /* set DPRAM pointer to data area
					  and read returned word           */
 mlx_out16_port(M044_RW16_DPR_POINTER_POS, M044_DPR_DATAS);
 ErrorStruct[micro_slot][1] = mlx_in16_port(M044_RW16_DPR_WORD);

 if(timeout == 0)
   {
    ErrorStruct[micro_slot][1] = 1;
   }

 if(ErrorStruct[micro_slot][1] != 0) return(0x40e0);
 return(0);
}

/*------------------------------------------------------<END OF M044_LIB.C>*/
