/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-DPM-12                **/
/**                                                                       **/
/**                             Version 1.D.                              **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1996-2000                      **/
/**                                                                       **/
/***************************************************************************/

#ifndef M_DPM_12
#define M_DPM_12


#ifdef ML8RT
   #include "ml8rtbib.h"
   #include "dos.h"
   #define  ML10API
   #define  MLXEXPORT     far pascal
   #define  mlx_out8_port(port, dat1)      outportb(port, dat1)
   #define  mlx_out16_port(port, dat1)     outport(port, dat1)
   #define  mlx_in8_port(port)             inportb(port)
   #define  mlx_in16_port(port)            inport(port)
   #define  mlx_get_task_number(pgm, inst) ml8rt_get_task_number(pgm, inst)

#else
   #ifdef _WIN32
      #include "MLXW32.H"
   #else
      #include "ML8BIB.H"
      #define  MLXEXPORT  EXPORT
   #endif
   #define mlx_read_eeprom ml8_read_eeprom_copy
   #define mlx_out8_port   ml8_out8_port
   #define mlx_out16_port  ml8_out16_port
   #define mlx_in8_port    ml8_in8_port
   #define mlx_in16_port   ml8_in16_port
   #define mlx_get_task_number(pgm, inst) ml8_get_task_number(pgm, inst)
#endif

#ifdef BORLAND_PASCAL
   #define STATIC static                /* for definition of arrays */
#else
   #define STATIC static far            /* for definition of arrays */
#endif

/****************************************************************************/
// Variable type definitions used by M044_MDD:

#ifndef byte
   #define byte unsigned char
#endif
#ifndef ulong
   #define ulong unsigned long
#endif

#ifdef ML8RT
   #ifndef ushort
      #define ushort unsigned int
   #endif
#else
   #ifndef ushort
      #define ushort unsigned short
   #endif
#endif

#ifdef _WIN32
        #pragma pack(push, SORCPACK, 1)     // for the 32-bit DLL (MSVC)
#else
        #pragma -a-                                // for dos, rt, win (bc)
#endif
// type definitions
typedef struct {
        ushort inp_ptr;
        ushort outp_ptr;
        ushort diag_ptr;
        ushort diag_len_ptr;
        ushort diag_cnt_ptr;
        byte inp_len;
        byte outp_len;
        byte inout;
        byte slave_type;
} m044_slv_datastr_type;

typedef struct {
        ushort slave_0_15;
        ushort slave_16_31;
        ushort slave_32_47;
        ushort slave_48_63;
        ushort slave_64_79;
        ushort slave_80_95;
        ushort slave_96_111;
        ushort slave_112_123;
} m044_slv_bitmap_type;

typedef struct {
        byte master_status;
        byte id_high;
        byte id_low;
        byte master_hw_version;
        byte master_fw_version;
        byte user_hw_version;
        byte user_fw_version;
} m044_mst_statusstr_type;

typedef struct {
        ushort component;
        ushort subcomponent;
        ushort status;
        ushort error_number;
        ushort detail;
} m044_mst_syserr_type;

#ifdef _WIN32
        #pragma pack(pop, SORCPACK)
#else
        #pragma -a
#endif



/****************************************************************************/
// Constants

#define ANZ_MOD 11    /* MODULAR-4/486: number of modul slots: 10+1 (slot 0 is not used) */
#define M044TASK_PROG_NO         0x329
#define M044_EEPROM_WORD_0       0x222C

#define _M044_NO_MORE_DATA			0
#define _M044_MORE_DATA          1
#define _M044_START_DATA         2

#define _M044_NO_CONS            1
#define _M044_CONS               2
#define _M044_FIXED_PTR          4
#define _M044_INC_PTR            8

#define _M044_ISR                                1
#define _M044_SMR                                2
#define _M044_LER                                3
#define _M044_IST                                5

#define _M044_SYNCHRONIZE        0
#define _M044_STOP               0x40
#define _M044_CLEAR              0x80
#define _M044_OPERATE            0xC0

#define _M044_DEFAULT_PAR        0x10
#define _M044_EEPROM_PAR         0x20
#define _M044_ENABLE_IRQ         0x01

#define _M044_BROADCAST          0x7F
#define _M044_FREEZE             0x08
#define _M044_UNFREEZE           0x04
#define _M044_SYNC               0x20
#define _M044_UNSYNC             0x10

/* SPB address macros (MBA offset) */

/* read/write position of DPRAM pointer */
#define M044_RW16_DPR_POINTER_POS      (ushort)(MBA[micro_slot]+0x00)
/* read/write interrupt select register */
#define M044_RW8_INT_SEL_REGISTER      (ushort)(MBA[micro_slot]+0x01)
/* read interrupt status register */
#define M044_R8_INT_STATUS_REGISTER    (ushort)(MBA[micro_slot]+0x05)
/* read/write serial mode select register */
#define M044_RW8_SER_MODE_SEL_REGISTER (ushort)(MBA[micro_slot]+0x02)
/* read/write LED register */
#define M044_RW8_LED_REGISTER          (ushort)(MBA[micro_slot]+0x03)
/* read/write IO pins */
#define M044_RW16_IO_PINS              (ushort)(MBA[micro_slot]+0x04)
/* write single IO pins */
#define M044_W8_IO_PINS                (ushort)(MBA[micro_slot]+0x05)
/* read/write IO pin mode register */
#define M044_RW16_IO_PIN_MODE_REGISTER (ushort)(MBA[micro_slot]+0x06)
/* read version of gate array */
#define M044_R8_GATE_ARRAY_VERSION     (ushort)(MBA[micro_slot]+0x07)
/* write reset of FPGA and ASPC2 */
#define M044_W8_RESET                  (ushort)(MBA[micro_slot]+0x07)
/* read/write timeout counter */
#define M044_RW16_TIMEOUT_COUNTER      (ushort)(MBA[micro_slot]+0x08)
/* read/write word to/from DPRAM */
#define M044_RW16_DPR_WORD             (ushort)(MBA[micro_slot]+0x10)
/* read/write byte to/from DPRAM */
#define M044_RW16_DPR_BYTE             (ushort)(MBA[micro_slot]+0x12)
/* read/write word to/from DPRAM with incrementation of DPRAM pointer */
#define M044_RW16_DPR_WORD_INC         (ushort)(MBA[micro_slot]+0x14)
/* read/write byte to/from DPRAM with incrementation of DPRAM pointer */
#define M044_RW16_DPR_BYTE_INC         (ushort)(MBA[micro_slot]+0x16)
/* write consistence request */
#define M044_W8_CONSISTENCE            (ushort)(MBA[micro_slot]+0x18)
/* exit consistence and read status */
#define M044_R8_CONSISTENCE            (ushort)(MBA[micro_slot]+0x18)
/* read status */
#define M044_R8_STATUS                 (ushort)(MBA[micro_slot]+0x19)

/* DPRAM positions of diagnostic channel */
#define M044_DPR_SLAVE_DIAG_LIST       0x3EC0
#define M044_DPR_MASTER_STATUS         0x3ED0
#define M044_DPR_DATA_TRANSFER_LIST		0x3EE0
#define M044_DPR_SYSTEM_ERROR          0x3EF0
#define M044_DPR_STATUS                0x3EFC

/* DPRAM positions of communication channel */
#define M044_DPR_DATAS                 0x3F00
#define M044_DPR_DATA_LENGTH           0x3FFA
#define M044_DPR_WATCHDOG              0x3FFB
#define M044_DPR_REPLY                 0x3FFC
#define M044_DPR_REPLY_SEMAPHORE       0x3FFD
#define M044_DPR_COMMAND               0x3FFE
#define M044_DPR_COMMAND_SEMAPHORE     0x3FFF

// Function number return codes of m044_get_last_error
#define GET_ERROR                  1
#define BIB_STARTUP                2
#define CONFIGURE                  3
#define GET_MASTER_MODE            4
#define DOWNLOAD                   5
#define DOWNLOAD_FILE              6
#define UPLOAD                     7
#define UPLOAD_FILE                8
#define SET_SLAVE_ADDRESS          9
#define RESTART                    10
#define REFRESH_SLAVE_DATASTRUCT   11
#define SET_MASTER_MODE            12
#define WATCHDOG_ENABLE            13
#define WATCHDOG_RETRIGGER         14
#define RESET_MASTER               15
#define HARD_RESET                 16
#define SET_SLAVE_BYTE             17
#define SET_SLAVE_WORD             18
#define SET_SLAVE_BLOCK            19
#define GET_SLAVE_BYTE             20
#define GET_SLAVE_WORD             21
#define GET_SLAVE_BLOCK            22
#define SET_SLAVE_COMMAND          23
#define GET_SLAVE_DATASTRUCT       24
#define GET_SLAVE_DIAGNOSIS_LIST   25
#define CHECK_SLAVE_DIAGNOSIS      26
#define GET_SLAVE_DIAGNOSIS        27
#define GET_DATA_TRANSFER_LIST     28
#define CHECK_SLAVE_ACTIVE         29
#define GET_MASTER_STATUS_STRUCT   30
#define GET_SYSTEM_ERROR_STRUCT    31
#define POLL_SYSTEM_ERROR          32
#define FW_TIMER                   33


#if defined(__cplusplus)
extern "C" {
#endif

/****************************************************************************
Function Prototypes
****************************************************************************/

ML10API ushort MLXEXPORT m044_bib_startup (void);
ML10API ushort MLXEXPORT m044_configure (byte micro_slot);
ML10API ushort MLXEXPORT m044_get_master_mode (byte micro_slot, byte *status);
ML10API ushort MLXEXPORT m044_download (byte micro_slot, void *data,
                                                                                                         byte data_size,  byte flag);
ML10API ushort MLXEXPORT m044_upload (byte micro_slot, void *data,
                                                                                                  byte *data_size, byte *flag);
ML10API ushort MLXEXPORT m044_set_slave_address (byte micro_slot,
                                                                                                                                 void *data, byte size);
ML10API ushort MLXEXPORT m044_restart (byte micro_slot, byte restart_par);
ML10API ushort MLXEXPORT m044_refresh_slave_datastruct (byte micro_slot);
ML10API ushort MLXEXPORT m044_set_master_mode (byte micro_slot, byte command);
ML10API ushort MLXEXPORT m044_watchdog_enable (byte micro_slot, ushort timeout);
ML10API ushort MLXEXPORT m044_watchdog_retrigger (byte micro_slot);
ML10API ushort MLXEXPORT m044_set_slave_byte (byte micro_slot, byte slave,
                                                                                                                         byte subchannel, byte data,
                                                                                                                         byte cons);
ML10API ushort MLXEXPORT m044_set_slave_word        (byte micro_slot, byte slave,
                                                                                                                         byte subchannel, ushort data,
                                                                                                                         byte cons);
ML10API ushort MLXEXPORT m044_set_slave_block (byte micro_slot, byte slave,
                                                                                                                          ushort offset, byte size,
                                                                                                                          void *data, byte cons);
ML10API ushort MLXEXPORT m044_get_slave_byte (byte micro_slot, byte slave,
                                                                                                                         byte subchannel, byte *data,
                                                                                                                         byte cons);
ML10API ushort MLXEXPORT m044_get_slave_word        (byte micro_slot, byte slave,
                                                                                                                         byte subchannel, ushort *data,
                                                                                                                         byte cons);
ML10API ushort MLXEXPORT m044_get_slave_block (byte micro_slot, byte slave,
                                                                                                                          ushort offset, byte size,
                                                                                                                          void *data, byte cons);
ML10API ushort MLXEXPORT m044_set_slave_command (byte micro_slot, byte command,
                                                                                                                                 byte group, byte slave);
ML10API ushort MLXEXPORT m044_get_slave_datastruct (byte micro_slot, byte slave,
                                                                                                                                         m044_slv_datastr_type *data);
ML10API ushort MLXEXPORT m044_get_slave_diagnosis_list (byte micro_slot,
                                                                                                                                                  m044_slv_bitmap_type *data);
ML10API ushort MLXEXPORT m044_check_slave_diagnosis (byte micro_slot, byte slave,
                                                                                                                                          byte *diaglen);
ML10API ushort MLXEXPORT m044_get_slave_diagnosis (byte micro_slot, byte slave,
                                                                                                                                        byte size, void *data);
ML10API ushort MLXEXPORT m044_get_data_transfer_list (byte micro_slot,
                                                                                                                                                m044_slv_bitmap_type *data);
ML10API ushort MLXEXPORT m044_check_slave_active (byte micro_slot, byte slave,
                                                                                                                                  byte *active);
ML10API ushort MLXEXPORT m044_get_master_status_struct (byte micro_slot,
                                                                                                                                                  m044_mst_statusstr_type *data);
ML10API ushort MLXEXPORT m044_get_system_error_struct (byte micro_slot,
                                                                                                                                                 m044_mst_syserr_type *data);
ML10API ushort MLXEXPORT m044_poll_system_error (byte micro_slot, ushort *syserr);


// Internal functions
ML10API void MLXEXPORT m044_set_dpram_pointer (byte micro_slot, ushort position);
ML10API ushort MLXEXPORT m044_get_dpram_pointer (byte micro_slot);
ML10API void MLXEXPORT m044_set_dpram_byte (byte micro_slot,
                                                                                                          byte data, byte ptr_mode);
ML10API void MLXEXPORT m044_set_dpram_word (byte micro_slot,
                                                                                                                  ushort data, byte ptr_mode);
ML10API void MLXEXPORT m044_set_dpram_block (byte micro_slot,
                                                                                                                        void *data, byte size);
ML10API byte MLXEXPORT m044_get_dpram_byte (byte micro_slot, byte ptr_mode);
ML10API ushort MLXEXPORT m044_get_dpram_word (byte micro_slot, byte ptr_mode);
ML10API void MLXEXPORT m044_get_dpram_block (byte micro_slot,
                                                                                                                        void *data, byte size);
ML10API ushort MLXEXPORT m044_hard_reset (byte micro_slot);
ML10API ushort MLXEXPORT m044_reset_master (byte micro_slot);
ML10API byte MLXEXPORT m044_get_fpga_version (byte micro_slot);
ML10API void MLXEXPORT m044_set_modul_register(byte micro_slot,
                                                                                                                          byte reg_type, byte data);
ML10API byte MLXEXPORT m044_get_modul_register (byte micro_slot, byte reg_type);
ML10API void MLXEXPORT m044_configure_io_pin        (byte micro_slot, ushort data);
ML10API ushort MLXEXPORT m044_get_io_pin_config(byte micro_slot);
ML10API void MLXEXPORT m044_set_io_pin (byte micro_slot, ushort data);
ML10API ushort MLXEXPORT m044_get_io_pin (byte micro_slot);
ML10API void MLXEXPORT m044_set_timeout_counter (byte micro_slot, ushort data);
ML10API ushort MLXEXPORT m044_get_timeout_counter(byte micro_slot);
ML10API void MLXEXPORT m044_set_cons (byte micro_slot, byte mode);
ML10API byte MLXEXPORT m044_clear_cons(byte micro_slot);
ML10API byte MLXEXPORT m044_get_modul_status(byte micro_slot);
ML10API ushort MLXEXPORT m044_download_file (byte micro_slot, const char *binary_file,
                                                                                                                        ushort *cnt);
ML10API ushort MLXEXPORT m044_upload_file (byte micro_slot, const char *binary_file,
                                                                                          ushort *uploaded);

ML10API ushort MLXEXPORT m044_get_error (byte micro_slot, ushort *data);
ML10API void MLXEXPORT m044_set_tasknum(ushort task);
ML10API ushort MLXEXPORT m044_fw_timer        (byte micro_slot, ushort timer);

#ifdef BORLAND_PASCAL
extern ML10API ushort MLXEXPORT m044_download_file_p (byte micro_slot, const char *binary_file,
                                                                                                                                 ushort *cnt);
extern ML10API ushort MLXEXPORT m044_upload_file_p (byte micro_slot, const char *binary_file,
                                                                                                           ushort *uploaded);
#endif

#if defined(__cplusplus)
} /*  end of block extern "C" */
#endif

#endif /* #ifndef M_DPM_12 */


