/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 MDD-Library for SPB module M-DPM-12            **/
/**                                                                       **/
/**                             Version 2.A.                              **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1996-2000                      **/
/**                                                                       **/
/***************************************************************************/
/* History:																						*/
/* Version 2.A - changes to generate a DLL for WIN32                			*/
/*             - changes so that the library can support more than one		*/
/*               MODULAR-4 board		                                       */
/*         		- m044_bib_startup changed: it detects now automatically    */
/*               the task number	of the M044TASK-program  				      */
/*             - m044_fw_timer did not work in RT programs                 */ 
/***************************************************************************/

#ifdef _WINDOWS
   #include <windows.h>
#endif

#include "m044_lib.h"

#ifndef ML8RT
	#include "io.h"
	#include "stdio.h"
	#include "fcntl.h"
#endif

#ifdef _WIN16
   HANDLE   hInst;
   #pragma argsused
   int far pascal LibMain(hInstance, wDataSeg, wHeapSize, lpszCmdLine)
      HANDLE  hInstance;                   /* instance of DLL */
      WORD    wDataSeg;                    /* data segment */
      WORD    wHeapSize;                   /* size of heap */
      LPSTR   lpszCmdLine;                 /* pointer to command line */
      {
         hInst = hInstance;
         if(wHeapSize > 0)
            UnlockData(0);
         return(1);
      }
#endif 

#ifdef ML8RT
   STATIC ushort tasknum = 0xFFFF;
#else
   STATIC USHORT tasknum[8] = {0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF};   
#endif


#ifndef ML8RT
   STATIC ushort ErrorStruct[ANZ_MOD][20];
#endif

STATIC ushort MBA[ANZ_MOD] = {0x000, 0x400, 0x500, 0x600, 0x700,
										0x420, 0x440, 0x460, 0x480, 0x4A0, 0x4C0};


/****************************************************************************
// m044_bib_startup
****************************************************************************/
ML10API ushort MLXEXPORT m044_bib_startup(void)
{
   ushort install_nr, t, card;
	ushort insize, result, outsize, error, outdata;

	outsize		= 0;
	error			= 0;
	install_nr	= 1;
#ifdef ML8RT
	if (tasknum == 0xFFFF) {
	   t = ml8rt_get_task_number(M044TASK_PROG_NO, &install_nr);
		if (install_nr == 1)
			tasknum = t;
	}
	ml8rt_call_func(tasknum, 2, &outsize, &outdata, 
						 0, &insize, &result, &error);
#else
   card = ml8_get_selected_card();
	if (tasknum[card] == 0xFFFF) {
	   t = ml8_get_task_number(M044TASK_PROG_NO, &install_nr);
		if (install_nr == 1)
			tasknum[card] = t;
	}
	ml8_call_func(tasknum[card], 2, outsize, &outdata, 
					  0, &insize, &result, (byte *)&error);
#endif
	return(error);
}

/****************************************************************************
// m044_configure
****************************************************************************/
ML10API ushort MLXEXPORT m044_configure(byte micro_slot)
{
	ushort insize, result, outsize, error, card;

	outsize = 1;
	error = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 3, 
					   outsize, &micro_slot,            
					   0, &insize, &result, (byte*)&error);
#else
	ml8rt_call_func (tasknum, 3, 
						  &outsize, &micro_slot,         
					     0, &insize, &result, &error);
#endif
	return(error);
}

/****************************************************************************
// m044_get_master_mode
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_master_mode(byte micro_slot, byte *status)
{
	ushort insize, outsize, error, card;

	outsize = 1;
	error = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 4, outsize, &micro_slot, 
					  1, &insize, status, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 4, &outsize, &micro_slot,
					    1, &insize, status, &error);
#endif
	return(error);
}

/****************************************************************************
// m044_restart
****************************************************************************/
ML10API ushort MLXEXPORT m044_restart(byte micro_slot, byte restart_par)
{
	ushort insize, outsize, error, result, card;
	byte out[2];

	out[0]	= micro_slot;
	out[1]	= restart_par;
	outsize	= 2;
	error		= 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 5, outsize, &out,                   
					  0,&insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 5, &outsize, &out,
					    0,&insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_refresh_slave_datastruct
****************************************************************************/
ML10API ushort MLXEXPORT m044_refresh_slave_datastruct(byte micro_slot)
{
	ushort insize, outsize, error, result, card;

	outsize = 1;
	error	= 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 6,
					   outsize, &micro_slot, 
					   0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 6,
						 &outsize, &micro_slot, 
						 0, &insize, &result, &error);
#endif
   return(error);
}


/****************************************************************************
// m044_get_slave_datastruct
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_slave_datastruct
	(byte micro_slot, byte slave, m044_slv_datastr_type *data)
{
	byte out[2];
	ushort insize, outsize, error, card;

	out[0]  = micro_slot;
	out[1]  = slave;
	error	  = 0;
	outsize = 2;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 7,
						outsize, &out, 
						sizeof(*data), &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 7,
						 &outsize, &out, 
						 sizeof(*data), &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_check_slave_active
****************************************************************************/
ML10API ushort MLXEXPORT m044_check_slave_active(byte micro_slot, byte slave, byte *active)
{
	byte out[2];
	ushort insize, outsize, error, card;

	out[0]	= micro_slot;
	out[1]	= slave;
	outsize	= 2;
	error		= 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 8,
						outsize, &out, 
						1, &insize, active, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 8,
						 &outsize, &out, 
						 1, &insize, active, &error);
#endif
	return(error);
}


/****************************************************************************
// 9                                                      m044_get_slave_byte
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_slave_byte
	(byte micro_slot, byte slave, byte subchannel, byte *data, byte cons)
{
	ushort insize, outsize, error, card;
	byte out[5];

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = subchannel;
	out[4] = cons;
	outsize = 5;
	error	= 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 9,
						outsize, &out, 
						1, &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 9,
						 &outsize, &out, 
						 1, &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// 10                                                     m044_get_slave_word
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_slave_word
	(byte micro_slot, byte slave, byte subchannel, ushort *data, byte cons)
{
	ushort insize, outsize, error, card;
	byte out[5];

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = subchannel;
	out[4] = cons;
	outsize = 5;
	error	= 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 10,
					  outsize, &out, 
					  2, &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 10,
		  			    &outsize, &out, 
					    2, &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_slave_block
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_slave_block
	(byte micro_slot, byte slave, ushort offset, byte size, void *data, byte cons)
{
	byte out[6];
	ushort insize, outsize, error, card;

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = (byte)offset;
	out[3] = (byte)(offset >> 8);
	out[4] = size;
	out[5] = cons;
	error	 = 0;
	outsize= 6;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 11,
					   outsize, &out,  
						size, &insize, data, (byte *)&error);
#else
   ml8rt_call_func(tasknum, 11,
						 &outsize, &out, 
						 size, &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_set_slave_byte
****************************************************************************/
ML10API ushort MLXEXPORT m044_set_slave_byte
	(byte micro_slot, byte slave, byte subchannel, byte data, byte cons)
{
	ushort insize, outsize, error, result, card;
	byte out[5];

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = subchannel;
	out[3] = data;
	out[4] = cons;
	error  = 0;
	outsize = 5;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 12,
						outsize, &out,  
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 12,
						 &outsize, &out, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_set_slave_word
****************************************************************************/
ML10API ushort MLXEXPORT m044_set_slave_word
	(byte micro_slot, byte slave, byte subchannel, ushort data, byte cons)
{
	ushort insize, outsize, error, result, card;
	byte out[6];

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = subchannel;
	out[3] = (byte)data;
	out[4] = (byte)(data >> 8);
	out[5] = cons;
	error  = 0;
	outsize = 6;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 13,
						outsize, &out, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 13,
						 &outsize, &out, 
						 0, &insize, &result, &error);
#endif
   return(error);
}


/****************************************************************************
// m044_set_slave_block
****************************************************************************/
ML10API ushort MLXEXPORT m044_set_slave_block
	(byte micro_slot, byte slave, ushort offset, byte size, void *data, byte cons)
{
	ushort i, insize, outsize, error, result, card;
	byte out[256];

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = (byte)offset;
	out[3] = (byte)(offset >> 8);
	out[4] = size;
	out[5] = cons;
	error  = 0;
	for(i = 0; i < size; i++)
		out[6 + i] = *((byte *)data + i);
	outsize = size + 6;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 14,
						outsize, &out, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 14,
						 &outsize, &out,  
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_set_master_mode
****************************************************************************/
ML10API ushort MLXEXPORT m044_set_master_mode(byte micro_slot, byte command)
{
	byte out[2];
	ushort insize, outsize, result, error, card;

	out[0] = micro_slot;
	out[1] = command;
	outsize = 2;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 15,
						outsize, &out, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 15,
						 &outsize, &out, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_watchdog_enable
****************************************************************************/
ML10API ushort MLXEXPORT m044_watchdog_enable(byte micro_slot, ushort timeout)
{
	ushort result, insize, outsize, error, card;
	byte out[3];

	out[0] = micro_slot;
	out[1] = (byte)timeout;
	out[2] = (byte)(timeout >> 8);
	outsize= 3;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 16,
						outsize, &out, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 16,
						 &outsize, &out, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_watchdog_retrigger
****************************************************************************/
ML10API ushort MLXEXPORT m044_watchdog_retrigger(byte micro_slot)
{
	ushort insize, outsize, result, error, card;

	outsize = 1;
	error   = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 17,
						outsize, &micro_slot, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 17,
						 &outsize, &micro_slot, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_set_slave_address
****************************************************************************/
ML10API ushort MLXEXPORT m044_set_slave_address
	(byte micro_slot, void *data, byte size)
{
ushort i, insize, outsize, error, result, card;
byte out[255];

	out[0] = micro_slot;
	out[1] = size;
	for(i = 0; i < size; i++)
		out[2 + i] = *((byte *)data + i);
	error   = 0;
	outsize = size + 2;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 18,
					  outsize, &out, 
					  0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 18,
						 &outsize, &out,  
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_set_slave_command
****************************************************************************/
ML10API ushort MLXEXPORT m044_set_slave_command
	(byte micro_slot, byte command, byte group, byte slave)
{
	byte out[4];
	ushort insize, outsize, error, result, card;

	out[0] = micro_slot;
	out[1] = command;
	out[2] = group;
	out[3] = slave;
	outsize = 4;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 19,
						outsize, &out, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 19,
						 &outsize, &out, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_slave_diagnosis_list
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_slave_diagnosis_list
        (byte micro_slot, m044_slv_bitmap_type *data)
{
  ushort insize, outsize, error, card;

	outsize = 1;
	error   = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 20,
						outsize, &micro_slot, 
						sizeof(*data), &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 20,
						 &outsize, &micro_slot, 
						 sizeof(*data), &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_check_slave_diagnosis
****************************************************************************/
ML10API ushort MLXEXPORT m044_check_slave_diagnosis(byte micro_slot, byte slave, byte* diaglen)
{
	byte out[2];
	ushort insize, outsize, error, card;

	out[0] = micro_slot;
	out[1] = slave;
	outsize = 2;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 21,
						outsize, &out, 
						1, &insize, diaglen, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 21,
						 &outsize, &out, 
						 1, &insize, diaglen, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_slave_diagnosis
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_slave_diagnosis
        (byte micro_slot, byte slave, byte size, void *data)
{
	byte out[3];
	ushort insize, outsize, error, card;

	out[0] = micro_slot;
	out[1] = slave;
	out[2] = size;
	outsize = 3;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 22,
					   outsize, &out, 
						size, &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 22,
						 &outsize, &out, 
						 size, &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_data_transfer_list
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_data_transfer_list
	(byte micro_slot, m044_slv_bitmap_type *data)
{
	ushort insize, outsize, error, card;

	outsize = 1;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 23,
						outsize, &micro_slot, 
						sizeof(*data), &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 23,
						 &outsize, &micro_slot, 
						 sizeof(*data), &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_master_status_struct
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_master_status_struct
	(byte micro_slot, m044_mst_statusstr_type *data)
{
	ushort insize, outsize, error, card;

	outsize = 1;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 24,
						outsize, &micro_slot, 
						sizeof(*data), &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 24,
						 &outsize, &micro_slot, 
						 sizeof(*data), &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_system_error_struct
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_system_error_struct
	(byte micro_slot, m044_mst_syserr_type *data)
{
	ushort insize, outsize, error, card;

	outsize = 1;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 25,
						outsize, &micro_slot, 
						sizeof(*data), &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 25,
						 &outsize, &micro_slot, 
						 sizeof(*data), &insize, data, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_poll_system_error
****************************************************************************/
ML10API ushort MLXEXPORT m044_poll_system_error(byte micro_slot, ushort *syserr)
{
	ushort insize, outsize, error, card;

	outsize = 1;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 26,
						outsize, &micro_slot,
						2, &insize, syserr, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 26,
						 &outsize, &micro_slot, 
						 2, &insize, syserr, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_download
****************************************************************************/
ML10API ushort MLXEXPORT m044_download
	(byte micro_slot, void *data, byte data_size, byte flag)
{
	ushort i, insize, outsize, error, result, card;
	byte out[204];
	static byte blocknum = 0;

	out[0] = micro_slot;
	out[1] = data_size;
	out[2] = flag;
	out[3] = blocknum;
	for(i = 0; i < data_size; i++)
		out[4 + i] = *((byte *)data + i);
	outsize = data_size + 4;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 27,
					  outsize, &out, 
					  0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 27,
						 &outsize, &out, 
						 0, &insize, &result, &error);
#endif
	blocknum++; 
	/* if finished, reset blocknum      */
	/* blocknum = 0 disables all other task-functions until last block */
	if(flag == _M044_NO_MORE_DATA) 
		blocknum = 0;
	if(error) 
		blocknum = 0;
	return(error);
}


/****************************************************************************
// m044_upload
****************************************************************************/
ML10API ushort MLXEXPORT m044_upload
	(byte micro_slot, void *data, byte *data_size, byte *flag)
{
	ushort insize, outsize, error, card;
	byte out[2], *ptr;

	out[0] = micro_slot;
	out[1] = *flag;
	outsize = 2;
	ptr = (byte *)data;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 28,
					  outsize, &out, 
					  252, &insize, ptr, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 28,
						 &outsize, &out, 
						 252, &insize, ptr, &error);
#endif
	*flag      = ptr[insize - 1];
	*data_size = ptr[insize - 2];
	return(error);
}

/****************************************************************************
// m044_hard_reset
****************************************************************************/
ML10API ushort MLXEXPORT m044_hard_reset(byte micro_slot)
{
	ushort insize, outsize, result, error, card;

	outsize = 1;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 29,
						outsize, &micro_slot, 
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 29,
						 &outsize, &micro_slot, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_hard_reset
****************************************************************************/
ML10API ushort MLXEXPORT m044_reset_master(byte micro_slot)
{
	ushort insize, outsize, result, error, card;

	outsize = 1;
	error  = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 30,
						outsize, &micro_slot,
						0, &insize, &result, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 30,
						 &outsize, &micro_slot, 
						 0, &insize, &result, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_error
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_error(byte micro_slot, ushort *data)
{
	ushort insize, outsize, error, card;

	outsize = 1;
	error   = 0;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func(tasknum[card], 31,
					  outsize, &micro_slot, 
					  20, &insize, data, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 31,
						 &outsize, &micro_slot,
						 20, &insize, data, &error);
#endif
	return(error);
}

/****************************************************************************
// m044_get_error
****************************************************************************/
ML10API ushort MLXEXPORT m044_fw_timer(byte micro_slot, ushort timer)
{
	ushort insize, outsize, error, card;
	byte out[3], in;

	out[0]  = micro_slot;
	out[1]  = (byte)timer;
	out[2]  = (byte)(timer >> 8);
	error   = 0;
	outsize = 3;
#ifndef ML8RT
   card = ml8_get_selected_card();
	ml8_call_func (tasknum[card], 32,
					   outsize, &out, 
						0, &insize, &in, (byte *)&error);
#else
	ml8rt_call_func(tasknum, 32,
						 &outsize, &out, 
						 0, &insize, &in, &error);
#endif
	return(error);
}


/****************************************************************************
// m044_get_fpga_version
****************************************************************************/
byte EXPORT m044_get_fpga_version(byte micro_slot)
{
	return mlx_in8_port(M044_R8_GATE_ARRAY_VERSION);
}


/****************************************************************************
// m044_set_modul_register
****************************************************************************/
void EXPORT m044_set_modul_register
	(byte micro_slot, byte reg_type, byte data)
{
	switch(reg_type) {
    case _M044_ISR :
       mlx_out8_port(M044_RW8_INT_SEL_REGISTER, data);
       break;
    case _M044_SMR :
       mlx_out8_port(M044_RW8_SER_MODE_SEL_REGISTER, data);
       break;
    case _M044_LER :
       mlx_out8_port(M044_RW8_LED_REGISTER, data);
       break;
   }
}


/****************************************************************************
// m044_get_modul_register
****************************************************************************/
byte EXPORT m044_get_modul_register(byte micro_slot, byte reg_type)
{
	byte data;

	switch(reg_type) {
		case _M044_ISR :
			data = mlx_in8_port(M044_RW8_INT_SEL_REGISTER);
			break;
		case _M044_IST :
			data = mlx_in8_port(M044_R8_INT_STATUS_REGISTER);
			break;
		case _M044_SMR :
			data = mlx_in8_port(M044_RW8_SER_MODE_SEL_REGISTER);
			break;
		case _M044_LER :
			data = mlx_in8_port(M044_RW8_LED_REGISTER);
			break;
	}
	return data;
}


/****************************************************************************
// m044_configure_io_pin
****************************************************************************/
void EXPORT m044_configure_io_pin(byte micro_slot, ushort data)
{
	mlx_out16_port(M044_RW16_IO_PIN_MODE_REGISTER, data);
}


/****************************************************************************
// m044_get_io_pin_config
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_io_pin_config(byte micro_slot)
{
	return mlx_in16_port(M044_RW16_IO_PIN_MODE_REGISTER);
}


/****************************************************************************
// m044_set_io_pin
****************************************************************************/
void EXPORT m044_set_io_pin(byte micro_slot, ushort data)
{
	mlx_out16_port(M044_RW16_IO_PINS, data);
}


/****************************************************************************
// m044_get_io_pin
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_io_pin(byte micro_slot)
{
	return(mlx_in16_port(M044_RW16_IO_PINS));
}


/****************************************************************************
// m044_set_timeout_counter
****************************************************************************/
void EXPORT m044_set_timeout_counter(byte micro_slot, ushort data)
{
	mlx_out16_port(M044_RW16_TIMEOUT_COUNTER, data);
}


/****************************************************************************
// m044_get_timeout_counter
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_timeout_counter(byte micro_slot)
{
	return(mlx_in16_port(M044_RW16_TIMEOUT_COUNTER));
}


/****************************************************************************
// m044_set_cons
****************************************************************************/
void EXPORT m044_set_cons(byte micro_slot, byte mode)
{
	mlx_out8_port(M044_W8_CONSISTENCE, mode);
}


/****************************************************************************
// m044_clear_cons
****************************************************************************/
byte EXPORT m044_clear_cons(byte micro_slot)
{
	return(mlx_in8_port(M044_R8_CONSISTENCE));
}


/****************************************************************************
// m044_get_modul_status
****************************************************************************/
byte EXPORT m044_get_modul_status(byte micro_slot)
{
	return(mlx_in8_port(M044_R8_STATUS));
}


/****************************************************************************
// m044_set_tasknum (not used anymore from version 2.A
****************************************************************************/
ML10API void MLXEXPORT m044_set_tasknum(ushort task)
{
#ifdef ML8RT
	tasknum = task;
#else
	ushort card;
   card = ml8_get_selected_card();
	tasknum[card] = task;
#endif
}

/****************************************************************************
						       m044_set_dpram_pointer
****************************************************************************/
void EXPORT m044_set_dpram_pointer(byte micro_slot, ushort position)
{
	mlx_out16_port(M044_RW16_DPR_POINTER_POS, position);
}


/****************************************************************************
						       m044_get_dpram_pointer
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_dpram_pointer(byte micro_slot)
{
	return(mlx_in16_port(M044_RW16_DPR_POINTER_POS));
}


/****************************************************************************
							  m044_set_dpram_byte
****************************************************************************/
void EXPORT m044_set_dpram_byte(byte micro_slot, byte data, byte ptr_mode)
{
	if(mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01) { // odd ??
		if(ptr_mode == _M044_FIXED_PTR)
		   mlx_out16_port(M044_RW16_DPR_BYTE, (ushort)(((ushort) data) << 8));
		else
		   mlx_out16_port(M044_RW16_DPR_BYTE_INC, (ushort)(((ushort) data) << 8));
	}
	else {                                   /* pointer is even */
		if(ptr_mode == _M044_FIXED_PTR)
		   mlx_out16_port(M044_RW16_DPR_BYTE, ((ushort) data));
		else
		   mlx_out16_port(M044_RW16_DPR_BYTE_INC, ((ushort) data));
	}
}


/****************************************************************************
							  m044_set_dpram_word
****************************************************************************/
void EXPORT m044_set_dpram_word(byte micro_slot, ushort data, byte ptr_mode)
{
	ushort dpram_ptr_pos, data_copy;

	if((dpram_ptr_pos = mlx_in16_port(M044_RW16_DPR_POINTER_POS)) & 0x01) { // odd ??
		data_copy = data;
		if(ptr_mode == _M044_FIXED_PTR) {
			 /* set low byte to odd address, set high byte to even address,
			    restore changed dpram pointer    */
			 mlx_out16_port(M044_RW16_DPR_BYTE_INC, (ushort)(((ushort) data_copy) << 8));
			 mlx_out16_port(M044_RW16_DPR_BYTE, (ushort)(((ushort) data) >> 8));
			 mlx_out16_port(M044_RW16_DPR_POINTER_POS, dpram_ptr_pos);
		}
		else { /* set low byte to odd address, set high byte to even address */
			 mlx_out16_port(M044_RW16_DPR_BYTE_INC, (ushort)(((ushort) data_copy) << 8));
			 mlx_out16_port(M044_RW16_DPR_BYTE_INC, (ushort)(((ushort) data) >> 8));
		}
   }
	else {                                   /* pointer is even                  */
		if(ptr_mode == _M044_FIXED_PTR)
			mlx_out16_port(M044_RW16_DPR_WORD, (ushort)data);
		else
			mlx_out16_port(M044_RW16_DPR_WORD_INC, (ushort)data);
   }
}


/****************************************************************************
							 m044_set_dpram_block
****************************************************************************/
void EXPORT m044_set_dpram_block(byte micro_slot, void *data, byte size)
{
	ushort	*ptr16;
	byte		*ptr8;

	ptr16 = (ushort *)data;
	ptr8  = (byte *)data;
	/* if DPRAM pointer odd, then write first byte        */
	if((mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01) && (size > 0)) {
		mlx_out16_port(M044_RW16_DPR_BYTE_INC, (ushort)(((ushort)*ptr16) << 8));
		size--;
		ptr8++;
		ptr16 = (ushort *)ptr8; // = (ushort *)(((byte *)data)++);
	}
	/* write words with auto-increment  */
	while(size > 1) {
		mlx_out16_port(M044_RW16_DPR_WORD_INC, *ptr16);
		ptr16++;
		size -= 2;
   }
   if(size)   /* if rest<>0, write last byte      */
		mlx_out16_port(M044_RW16_DPR_BYTE_INC, *ptr16);
}



/****************************************************************************
// m044_get_dpram_byte
****************************************************************************/
byte EXPORT m044_get_dpram_byte(byte micro_slot, byte ptr_mode)
{
	ushort data;

	if(mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01) {  /* pointer is odd                   */
		if(ptr_mode == _M044_FIXED_PTR)
			data = mlx_in16_port(M044_RW16_DPR_BYTE) >> 8;
		else
			data = mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8;
	}
	else {                                   /* pointer is even                  */
		if(ptr_mode == _M044_FIXED_PTR)
			data = mlx_in16_port(M044_RW16_DPR_BYTE);
		else
			data = mlx_in16_port(M044_RW16_DPR_BYTE_INC);
	}
	return((byte)data);
}


/****************************************************************************
// m044_get_dpram_word
****************************************************************************/
ML10API ushort MLXEXPORT m044_get_dpram_word(byte micro_slot, byte ptr_mode)
{
	ushort data, data_low, dpram_ptr_pos;

	if((dpram_ptr_pos = mlx_in16_port(M044_RW16_DPR_POINTER_POS)) & 0x01) { /* pointer is odd                   */
		if(ptr_mode == _M044_FIXED_PTR) {  /* get low byte from odd position, 
			get high byte from even address, restore changed dpram pointer  */
			data_low = mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8;
			data = (mlx_in16_port(M044_RW16_DPR_BYTE) << 8) | data_low;
			mlx_out16_port(M044_RW16_DPR_POINTER_POS, dpram_ptr_pos);
      }
		else {/* get low byte from odd position, get high byte from even address  */
			data_low = mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8;
			data = (mlx_in16_port(M044_RW16_DPR_BYTE_INC) << 8) | data_low;
      }
   }
	else {                                   /* pointer is even                  */
		if(ptr_mode == _M044_FIXED_PTR)
			data = mlx_in16_port(M044_RW16_DPR_WORD);
		else
			data = mlx_in16_port(M044_RW16_DPR_WORD_INC);
   }
	return(data);
}


/****************************************************************************
m044_get_dpram_block
****************************************************************************/
void EXPORT m044_get_dpram_block(byte micro_slot, void *data, byte size)
{
	ushort *ptr16;
	byte	 *ptr8;
	/* if DPRAM pointer odd, then read first byte                    */
	if((mlx_in16_port(M044_RW16_DPR_POINTER_POS) & 0x01) && (size > 0)) {
		ptr8 = (byte *)data;
		*ptr8 = (byte)(mlx_in16_port(M044_RW16_DPR_BYTE_INC) >> 8);
		size--;
		ptr8++;
		data = (void *)ptr8;
	}
	/* read words with auto-increment   */
	ptr16 = (ushort *)data;
	while(size > 1) {
		*ptr16 = mlx_in16_port(M044_RW16_DPR_WORD_INC);
		ptr16++;
		size -= 2;
	}
	/* if rest <> 0, read last byte     */
	if(size)	{
		ptr8 = (byte *)ptr16;
		*ptr8 = (byte)mlx_in16_port(M044_RW16_DPR_BYTE_INC);
	}
}


/*######################################################################*/
/* B I N A R Y   F I L E   T R A N S F E R  (PC-programs only)          */
/*######################################################################*/

#ifndef ML8RT

/****************************************************************************
							   m044_download_file
****************************************************************************/
ML10API ushort MLXEXPORT m044_download_file
	(byte micro_slot, const char *binary_file, ushort *downloaded)
{
	ushort error;
#ifndef BORLAND_PASCAL
	ushort fsize, i;
	int handle, size;
	byte data[250];
#endif

	ErrorStruct[micro_slot][0] = DOWNLOAD_FILE;
	*downloaded = 0;
#ifndef BORLAND_PASCAL
	if((handle = open(binary_file, O_RDONLY | O_BINARY)) == -1) 
		return(0x40e0);
	fsize = (ushort)filelength(handle);
	for(i = 0; i < ((fsize / 200) + 1); i++) {
		size = read(handle, data, 200);
		if(size == 200) 
			error = m044_download(micro_slot, data, (byte)size, _M044_MORE_DATA);
		else 
			error = m044_download(micro_slot, data, (byte)size, _M044_NO_MORE_DATA);
		*downloaded += size;
	}
	close(handle);
#else                                  /* BORLAND_PASCAL                   */
	ErrorStruct[micro_slot][0] = DOWNLOAD_FILE;
	error = m044_download_file_p(micro_slot, binary_file, downloaded);
#endif
	return(error);
}


/****************************************************************************
							     m044_upload_file
****************************************************************************/
ML10API ushort MLXEXPORT m044_upload_file
	(byte micro_slot, const char *binary_file, ushort *uploaded)
{
	ushort error = 0;
#ifndef BORLAND_PASCAL
	byte binary[255];
	byte flag, size;
	FILE *f;
	ushort len;
#endif

	ErrorStruct[micro_slot][0] = UPLOAD_FILE;
	*uploaded = 0;
#ifndef BORLAND_PASCAL
	if((f = fopen(binary_file, "wb")) != NULL) {
		flag = _M044_START_DATA;
		do {
			error = m044_upload(micro_slot, &binary, &size, &flag);
			if(error) 
				return(error);
			len = (byte) fwrite(&binary, 1, size, f);
			*uploaded = *uploaded + len;
			if(len != size) {
				ErrorStruct[micro_slot][1] = 1;
				error = 0x40e0;
			}
		}
		while((flag == _M044_MORE_DATA) && (!error));
	}
	else {
		ErrorStruct[micro_slot][1] = 1;
		error = 0x40e0;
	}
	fclose(f);
#else                                  /* BORLAND_PASCAL                   */
	error = m044_upload_file_p(micro_slot, binary_file, uploaded);
#endif
	return(error);
}

#endif // IFNDEF ML8RT
