{******************************************************************************}
{*                                                                            *}
{* Sample program for M-DPM-12 under Windows NT / 9x / 2000                   *}
{*                                                                            *}
{* This program demonstrates how to use the module library in a delphi program*}
{* It was tested with Delphi 5.0 under Windows NT 4.0                         *}
{*                                                                            *}
{* The left half of the window includes general functions of the module       *}
{* that do not depend on any specific bus configuration.                      *}
{*                                                                            *}
{* The right half of the window contains some functions that correspond  to   *}
{* a specific Profibus configuration with two slaves (address 4 and 5).       *}
{* The configuration is stored in the binary file MyBusCfg.2BF which was      *}
{* generated with the COM Profibus Software from SIEMENS.                     *}
{* The configuration consists of one M-DPM-12 as master,                      *}
{* one ET200B-4AI with 4 analog inputs as a slave with bus address 5,         *}
{* one ET200B-4AO with 4 analog outputs as a slave with bus address 4.        *}
{*                                                                            *}
{* Notes                                                                      *}
{* The binary bus configuration file should be downloaded only once.          *}
{* It is stored in the modules Flash memory then and will be activated        *}
{* automatically after every power-up.                                        *}
{*                                                                            *}
{* Copyright SORCUS Computer GmbH, Heidelberg  2000                           *}
{******************************************************************************}


unit sample;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MLXW32, MLXTYPE, M044_MDD, ExtCtrls;

type
  TForm1 = class(TForm)
    StartButton: TButton;
    VersionInfo: TMemo;
    ReadAINButton: TButton;
    MasterStatusSelect: TRadioGroup;
    DownloadButton: TButton;
    SlaveInfo5Button: TButton;
    InputDisplay: TMemo;
    SlaveDiagButton: TButton;
    TransferListButton: TButton;
    MasterStatusButton: TButton;
    ReadAIN1Button: TButton;
    AoutControlBar: TScrollBar;
    SlaveInfo4Button: TButton;
    LED2CheckBox: TCheckBox;
    LED1CheckBox: TCheckBox;
    Label1: TLabel;
    procedure StartButtonClick(Sender: TObject);
    procedure ReadAINButtonClick(Sender: TObject);
    procedure DownloadButtonClick(Sender: TObject);
    procedure MasterStatusSelectClick(Sender: TObject);
    procedure SlaveInfo5ButtonClick(Sender: TObject);
    procedure SlaveDiagButtonClick(Sender: TObject);
    procedure TransferListButtonClick(Sender: TObject);
    procedure MasterStatusButtonClick(Sender: TObject);
    procedure ReadAIN1ButtonClick(Sender: TObject);
    procedure AoutControlBarChange(Sender: TObject);
    procedure SlaveInfo4ButtonClick(Sender: TObject);
    procedure LED2CheckBoxClick(Sender: TObject);
    procedure LED1CheckBoxClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;
  tasknum: WORD;
  micro_slot: BYTE;
  slave4AI, slave4AO : BYTE;

const
  BINFILE  : STRING = 'MyBusCfg.2bf';
  M044TASK : STRING = '..\..\..\m044task\m044task.exe';

implementation

{$R *.DFM}

(**************************************************************************)
(* This function checks if the parameter 'error' indicates an error       *)
(**************************************************************************)

FUNCTION display_error (error: WORD; text : STR80): WORD;
var
   errormsg: str80;
   errordata : Array[0..10] of word;
BEGIN
   if (error <> 0) then begin
      errormsg := ' Error ' + IntToHex(error, 2) + ' in ' + text;
      if (error = $40) then begin      { $40 is the error code for an error
                                         detected by a function of m044task.exe }
         error := m044_get_error(micro_slot, errordata[0]);
         if (error <> 0) then
            errormsg := errormsg + ' (error in m044_get_error)'
         else
            errormsg := errormsg + ' (Function ' + IntToStr(errordata[0]) + ', Diagnosis ' + IntToStr(errordata[1]) +') ';
      end;
      ShowMessage(errormsg);
      display_error := 1;
   end
   else
      display_error := 0;
end;




procedure TForm1.StartButtonClick(Sender: TObject);
VAR
   error     : WORD;
   version, date: LONGINT;
   osxstr, datestr: str80;
   prog_info : array[0..3] of char;
   errormsg  : STR80;
   mode      : byte;
BEGIN
   osxstr     := 'ROM';
   micro_slot := 4;
   tasknum    := micro_slot;
   slave4AI   := 5;         // address of slave
   slave4AO   := 4;         // address of slave
   ml8_bib_startup(LANGUAGE_PASCAL);
   error := ml8_reset(1, $380, 1, 7, 0, NIL, osxstr, 20);
   if error <> 0 then begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in reset';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;
   error := ml8_transfer_and_install(M044TASK, 0, tasknum, $329, 0, $180, 0);
   if error <> 0 then begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in transfer of m049task';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;

   error := m044_bib_startup;
   if (display_error(error, 'm044_bib_startup') <> 0) then
      exit;

   { get version info }
   ml8_get_osx_version(version, date);
   ml8_create_version_string(osxstr, version);
   ml8_create_date_string(datestr, date);
   Form1.VersionInfo.Lines.Add('OsX ' + osxstr + datestr);
   ml8_get_lib_version(version, date);
   ml8_create_version_string(osxstr, version);
   ml8_create_date_string(datestr, date);
   Form1.VersionInfo.Lines.Add('Library ' + osxstr + ' ' + datestr);
   ml8_get_mlxdrv_version(version, date);
   ml8_create_version_string(osxstr, version);
   ml8_create_date_string(datestr, date);
   Form1.VersionInfo.Lines.Add('MLXDRV.SYS ' + osxstr + ' ' + datestr);
   ml8_get_task_info(tasknum, 4, LONGINT(prog_info));
   Form1.VersionInfo.Lines.Add('M044Task ' + prog_info[2] + '.' + prog_info[3] +
                               ' installed as task no ' + IntToStr(tasknum));
   Form1.VersionInfo.Lines.Add('M-DPM-12-Settings: Slot: ' + IntToStr(micro_slot));
   mode := m044_get_fpga_version(micro_slot);
   Form1.VersionInfo.Lines.Add('M-DPM-12 FPGA-Version ' + IntToHex(mode, 2));

   error := m044_configure(micro_slot);
   if (display_error(error, 'm044_configure') <> 0) then
      exit;
   error := m044_get_master_mode (micro_slot, mode);
   if (display_error(error, 'm044_get_master_mode') <> 0) then
      exit;
   if (mode = _M044_STOP) then
      Form1.MasterStatusSelect.ItemIndex := 0
   else
      if (mode <> _M044_STOP) then begin
         errormsg := 'Wrong Master Mode (' + IntToHex(mode, 1) + ')';
         ShowMessage(errormsg);
      end;
end;

procedure TForm1.DownloadButtonClick(Sender: TObject);
VAR
   error     : WORD;
   errormsg  : STR80;
   mode      : byte;
   cnt       : word;
begin
   error := m044_download_file(micro_slot, BINFILE, cnt);
   if (display_error(error, 'm044_download_file') <> 0) then
      exit;
   Sleep(500);
   error := m044_reset_master(micro_slot);
   if (display_error(error, 'm044_reset_master') <> 0) then
      exit;
   Sleep(500);
   error := m044_get_master_mode (micro_slot, mode);
   if (display_error(error, 'm044_get_master_mode') <> 0) then
      exit;
   if (mode <> _M044_STOP) then begin
      errormsg := 'Master not in Stop Mode (' + IntToHex(mode, 1) + ')';
      ShowMessage(errormsg);
   end;
end;

procedure TForm1.MasterStatusSelectClick(Sender: TObject);
var
   mode   : BYTE;
   error  : WORD;
begin
   case MasterStatusSelect.ItemIndex of
      0: mode := _M044_STOP;
      1: mode := _M044_OPERATE;
      2: mode := _M044_CLEAR
   else exit;
   end;
   error := m044_set_master_mode (micro_slot, mode);
   if (display_error(error, 'm044_set_master_mode') <> 0) then
      exit;
end;

procedure TForm1.SlaveInfo5ButtonClick(Sender: TObject);
VAR
   error  : WORD;
   data   : m044_slv_datastr_type;
   msg    : STRING;
begin
   error := m044_refresh_slave_datastruct(micro_slot);
   if (display_error(error, 'm044_refresh_slave_datastruct') <> 0) then
      exit;
   error := m044_get_slave_datastruct(micro_slot, slave4AI, data);
   if (display_error(error, 'm044_get_slave_datastruct') <> 0) then
      exit;
   msg := 'Slave Address ' + IntToStr(slave4AI) + ' (Type ' + IntToStr(data.slave_type) + ')';
   msg := msg + chr(13) + 'InpLength = ' + IntToStr(data.inp_len) + ' OutpLength = ' + IntToStr(data.outp_len);
   ShowMessage(msg);
end;

procedure TForm1.ReadAINButtonClick(Sender: TObject);
var
   error, i : WORD;
   msg      : STRING;
   data     : Array[0..4] of Smallint;
begin
   error := m044_refresh_slave_datastruct(micro_slot);
   if (display_error(error, 'm044_refresh_slave_datastruct') <> 0) then
      exit;
   error := m044_get_slave_block (micro_slot, slave4AI, 0, 8, data, _M044_CONS);
   if (display_error(error, 'm044_get_slave_block') <> 0) then
      exit;
   msg := '';
   for i:=0 to 3 do
       msg := msg + ' ' + IntToStr(data[i]);
   Form1.InputDisplay.Lines.Add(msg);
end;


procedure TForm1.SlaveDiagButtonClick(Sender: TObject);
var
   error, i : WORD;
   msg      : STRING;
   data     : m044_slv_bitmap_type;
   w        : ^WORD;
begin
   error := m044_get_slave_diagnosis_list (micro_slot, data);
   if (display_error(error, 'm044_get_slave_diagnosis_list') <> 0) then
      exit;
   msg := '';
   w := @data.slave_0_15;
   i := 0;
   while (i < 128) do begin
      msg := msg + 'Slave ' + IntToStr(i) + '-' + IntToStr(i+15) + ': ' + IntToHex(w^, 4) + chr(13);
      Inc(w);
      i := i + 16;
   end;
   ShowMessage(msg);
end;

procedure TForm1.TransferListButtonClick(Sender: TObject);
var
   error, i : WORD;
   msg      : STRING;
   data     : m044_slv_bitmap_type;
   w        : ^WORD;
begin
   error := m044_get_data_transfer_list (micro_slot, data);
   if (display_error(error, 'm044_get_data_transfer_list') <> 0) then
      exit;
   msg := '';
   w := @data.slave_0_15;
   i := 0;
   while (i < 128) do begin
      msg := msg + 'Slave ' + IntToStr(i) + '-' + IntToStr(i+15) + ': ' + IntToHex(w^, 4) + chr(13);
      Inc(w);
      i := i + 16;
   end;
   ShowMessage(msg);
end;

procedure TForm1.MasterStatusButtonClick(Sender: TObject);
var
   error    : WORD;
   msg      : STRING;
   data     : m044_mst_statusstr_type;
begin
   error := m044_get_master_status_struct (micro_slot, data);
   if (display_error(error, 'm044_get_master_status_struct') <> 0) then
      exit;
   msg := 'Status = ' + IntToHex(data.master_status, 2) + chr(13);
   msg := msg + 'id = ' + IntToStr(data.id_high * $100 + data.id_low) + chr(13);
   msg := msg + 'HW-Version = ' + IntToStr(data.master_hw_version) + chr(13);
   msg := msg + 'FW-Version = ' + IntToStr(data.master_fw_version) + chr(13);
   msg := msg + 'User-HW-Version = ' + IntToStr(data.user_hw_version) + chr(13);
   msg := msg + 'User-FW-Version = ' + IntToStr(data.user_fw_version);
   ShowMessage(msg);

end;

procedure TForm1.ReadAIN1ButtonClick(Sender: TObject);
var
   error    : WORD;
   msg      : STRING;
   data     : Smallint;
begin
   error := m044_refresh_slave_datastruct(micro_slot);
   if (display_error(error, 'm044_refresh_slave_datastruct') <> 0) then
      exit;
   error := m044_get_slave_word (micro_slot, slave4AI, 1, Word(data), _M044_CONS);
   if (display_error(error, 'm044_get_slave_word') <> 0) then
      exit;
   msg := ' ' + IntToStr(data);
   Form1.InputDisplay.Lines.Add(msg);
end;

procedure TForm1.AoutControlBarChange(Sender: TObject);
Var
   error : Word;
   Aout  : Integer;
begin
   error := m044_refresh_slave_datastruct(micro_slot);
   if (display_error(error, 'm044_refresh_slave_datastruct') <> 0) then
      exit;
   Aout := AoutControlBar.Position;
   Aout := Integer(Aout * $FFFF);
   Aout := Aout div 100;
   error := m044_set_slave_word (micro_slot, slave4AO, 0, Aout, _M044_CONS);
   if (display_error(error, 'm044_set_slave_word') <> 0) then
      exit;
end;

procedure TForm1.SlaveInfo4ButtonClick(Sender: TObject);
VAR
   error  : WORD;
   data   : m044_slv_datastr_type;
   msg    : STRING;
begin
   error := m044_refresh_slave_datastruct(micro_slot);
   if (display_error(error, 'm044_refresh_slave_datastruct') <> 0) then
      exit;
   error := m044_get_slave_datastruct(micro_slot, slave4AO, data);
   if (display_error(error, 'm044_get_slave_datastruct') <> 0) then
      exit;
   msg := 'Slave Address ' + IntToStr(slave4AO) + ' (Type ' + IntToStr(data.slave_type) + ')';
   msg := msg + chr(13) + 'InpLength = ' + IntToStr(data.inp_len) + ' OutpLength = ' + IntToStr(data.outp_len);
   ShowMessage(msg);
end;

procedure TForm1.LED2CheckBoxClick(Sender: TObject);
var
   LED : Byte;
begin
   if (LED2CheckBox.Checked) then
      LED := $A
   else
      LED := $2;
   m044_set_modul_register(micro_slot, _M044_LER, LED);
end;

procedure TForm1.LED1CheckBoxClick(Sender: TObject);
var
   LED : Byte;
begin
   if (LED1CheckBox.Checked) then
      LED := $5
   else
      LED := $1;
   m044_set_modul_register(micro_slot, _M044_LER, LED);
end;

end.
