{$C MOVEABLE DEMANDLOAD DISCARDABLE}
(***************************************************************************)
(**                                                                       **)
(**              MODULAR-4 library for SPB module M-DPM-12                **)
(**                                                                       **)
(**                             Version 2.A.                              **)
(**                                                                       **)
(**               (c) SORCUS Computer GmbH 1995-2000                      **)
(**                                                                       **)
(***************************************************************************)

unit m044_mdd;

interface

uses
{$IFDEF WINDOWS}
   {$IFDEF _WIN32}
      mlxw32, mlxtype;          (*Include library for MODULAR-4/486 under Windows 32 Bit*)
   {$ELSE}
      wml8bib;         (*Include library for MODULAR-4/486 under Windows 16 Bit*)
   {$ENDIF}
{$else}
   {$IFDEF ML8RT}
      ml8rtbib;        (*Include library for MODULAR-4/486 realtime programs*)
   {$ELSE}
      ml8bib;          (*Include library for MODULAR-4/486 under DOS*)
   {$ENDIF}
{$ENDIF}

const

 _M044_NO_MORE_DATA   =        0;
 _M044_MORE_DATA      =        1;
 _M044_START_DATA     =        2;

 _M044_NO_CONS               =        1;
 _M044_CONS               =        2;
 _M044_FIXED_PTR      =        4;
 _M044_INC_PTR               =        8;

 _M044_ISR               =        1;
 _M044_SMR               =        2;
 _M044_LER               =        3;
 _M044_IST               =        5;

 _M044_SYNCHRONIZE    =        0;
 _M044_STOP               =        $40;
 _M044_CLEAR               =        $80;
 _M044_OPERATE               =        $C0;

 _M044_DEFAULT_PAR    =        $10;
 _M044_EEPROM_PAR     =        $20;
 _M044_ENABLE_IRQ     =        $01;

 _M044_BROADCAST      =        $7F;
 _M044_FREEZE               =        $08;
 _M044_UNFREEZE       =        $04;
 _M044_SYNC               =        $20;
 _M044_UNSYNC               =        $10;

type
m044_slv_datastr_type = RECORD
         inp_ptr      : WORD;
         outp_ptr     : WORD;
         diag_ptr     : WORD;
         diag_len_ptr : WORD;
         diag_cnt_ptr : WORD;
         inp_len      : BYTE;
         outp_len     : BYTE;
         inout        : BYTE;
         slave_type   : BYTE;
        END;


 m044_slv_bitmap_type = RECORD
         slave_0_15    : WORD;
         slave_16_31   : WORD;
         slave_32_47   : WORD;
         slave_48_63   : WORD;
         slave_64_79   : WORD;
         slave_80_95   : WORD;
         slave_96_111  : WORD;
         slave_112_123 : WORD;
        END;


 m044_mst_statusstr_type = RECORD
         master_status       : BYTE;
         id_high             : BYTE;
         id_low              : BYTE;
         master_hw_version   : BYTE;
         master_fw_version   : BYTE;
         user_hw_version     : BYTE;
         user_fw_version     : BYTE;
        END;


 m044_mst_syserr_type = RECORD
         component : WORD;
         subcomponent: WORD;
         status: WORD;
         error_number: WORD;
         detail: WORD;
        END;



(* m044_lib_startup
   ... initializes this modul library.
*)
FUNCTION m044_bib_startup : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_configure
   ... configures a modul in dependency of the eeprom presets.
*)
FUNCTION m044_configure        (micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_master_mode
   ... reads the current master mode for coordination during modul startup.
   If the master mode is _M044_STOP, you can start the communication with
   the modul.
*)
FUNCTION m044_get_master_mode (micro_slot : BYTE; VAR status : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_download
   ... transfers binary datas from host to modul. The parameter 'flag'
   specifies, if more datas must be transfered or not:
   flag = _M044_MORE_DATAS    : more datas to transfer
   flag = _M044_NO_MORE_DATAS : no more datas to transfer
*)
FUNCTION m044_download (micro_slot : BYTE; VAR data; data_size, flag : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_upload
   ... transfers binary datas from modul to host. The flag buffer,
   referenced by pointer 'flag', specifies, if more datas must be
   transfered or not:
   flag buffer = _M044_MORE_DATAS    : more datas to transfer
   flag buffer = _M044_NO_MORE_DATAS : no more datas to transfer
*)
FUNCTION m044_upload (micro_slot : BYTE; VAR data; VAR data_size : BYTE; VAR flag : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_slave_address
   ... sets the address of a slave.
*)
FUNCTION m044_set_slave_address (micro_slot : BYTE; VAR data; size : BYTE ) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_restart
   ... resets the master software on the modul. Restart is always necessary
   after downloading or setting a slave address.
*)
FUNCTION m044_restart(micro_slot, restart_par : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_refresh_slave_datastruct
   ... transfers the data structure informations of each slave into the
   data structure buffer of the library. Calling this function is always
   necessary after restart.
*)
FUNCTION m044_refresh_slave_datastruct (micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_master_mode
   ... controls the master mode. The parameter 'command' specifies the
   mode to set:
   command = _M044_SYNCHRONIZE : synchronize host and master
   command = _M044_CLEAR       : sets mode CLEAR
   command = _M044_STOP        : sets mode STOP
   command = _M044_OPERATE     : sets mode OPERATE
*)
FUNCTION m044_set_master_mode (micro_slot, command : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_watchdog_enable
   ... enables the watchdog of the master.
*)
FUNCTION m044_watchdog_enable (micro_slot : BYTE; timeout : WORD) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_trigger_watchdog
   ... retriggers the watchdog of the master.
*)
FUNCTION m044_watchdog_retrigger (micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_slave_byte
   ... writes a byte to a subchannel of a slave. The parameter 'cons'
   specifies the data consistence:
   cons = _M044_NO_CONS : without consistence
   cons = _M044_CONS    : with consistence        (not valid for byteaccess)
*)
FUNCTION m044_set_slave_byte (micro_slot, slave, subchannel, data, cons : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_slave_word
   ... writes a word to a subchannel of a slave. The parameter 'cons'
   specifies the data consistence:
   cons = _M044_NO_CONS : without consistence
   cons = _M044_CONS    : with consistence
*)
FUNCTION m044_set_slave_word (micro_slot, slave, subchannel : BYTE; data : WORD; cons : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_slave_block
   ... writes a data block to the data channel of a slave, beginning with
   subchannel 0. The parameter 'cons' specifies the data consistence:
   cons = _M044_NO_CONS : without consistence
   cons = _M044_CONS    : with consistence
*)
FUNCTION m044_set_slave_block (micro_slot, slave : BYTE; offset : WORD;
                               size: BYTE; VAR data; cons : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_slave_byte
   ... reads a byte from a subchannel of a slave. The parameter 'cons'
   specifies the data consistence:
   cons = _M044_NO_CONS : without consistence
   cons = _M044_CONS    : with consistence
*)
FUNCTION m044_get_slave_byte (micro_slot, slave, subchannel : BYTE;
                              VAR data :BYTE; cons : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_slave_word
   ... reads a word from a subchannel of a slave. The parameter 'cons'
   specifies the data consistence:
   cons = _M044_NO_CONS : without consistence
   cons = _M044_CONS    : with consistence
*)
FUNCTION m044_get_slave_word (micro_slot, slave, subchannel : BYTE;
                              VAR data : WORD; cons : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_slave_block
   ... reads a data block from the data channel of a slave. begining with
   subchannel 0. The parameter 'cons' specifies the data consistence:
   cons = _M044_NO_CONS : without consistence
   cons = _M044_CONS    : with consistence
*)
FUNCTION m044_get_slave_block (micro_slot, slave : BYTE; offset : WORD;
                               size : BYTE; VAR data; cons : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_slave_command
   ... gives a slave an order:
   command = _M044_FREEZE   : freeze the inputs
   command = _M044_UNFREEZE : finish freezing the inputs
   command = _M044_SYNC     : synchronize the outputs
   command = _M044_UNSYNC   : finish synchronizing the outputs
*)
FUNCTION m044_set_slave_command        (micro_slot, command, group, slave : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_slave_datastruct
   ... reads a datastruct from the datastruct buffer of the library.
*)
FUNCTION m044_get_slave_datastruct (micro_slot, slave : BYTE;
                                    VAR data : m044_slv_datastr_type) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_diagnosis_list
   ... reads the slave diagnosis list.
*)
FUNCTION m044_get_slave_diagnosis_list (micro_slot : BYTE;
                                        VAR data : m044_slv_bitmap_type) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_check_slave_diagnosis
   ... checks, if a slave has sent any diagnosis messages. If it has sent
   a message, the function returns the length of this message. Otherwise
   length 0 will be returned.
*)
FUNCTION m044_check_slave_diagnosis (micro_slot, slave : BYTE; VAR diaglen : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_slave_diagnosis
   ... reads the diagnosis data block of a slave.
*)
FUNCTION m044_get_slave_diagnosis (micro_slot, slave, size : BYTE; VAR data) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_data_transfer_list
   ... reads the data transfer list.
*)
FUNCTION m044_get_data_transfer_list(micro_slot : BYTE; VAR data : m044_slv_bitmap_type) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_check_slave_active
   ... checks, if a slave is active.
*)
FUNCTION m044_check_slave_active (micro_slot, slave : BYTE; VAR active : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_master_status_struct
   ... reads the master status structure.
*)
FUNCTION m044_get_master_status_struct(micro_slot : BYTE; VAR data : m044_mst_statusstr_type) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_system_error_struct
   ... reads the system error structure.
*)
FUNCTION m044_get_system_error_struct(micro_slot : BYTE; VAR data : m044_mst_syserr_type) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_poll_system_error
   ... reads word 0 of the system error structure to poll the system error.
   It`s not necessary, if interrupt (see restart) is enabled
*)
FUNCTION m044_poll_system_error (micro_slot : BYTE; VAR syserr : WORD) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_dpram_pointer
   ... places the DPRAM pointer.
*)
PROCEDURE m044_set_dpram_pointer (micro_slot : BYTE; position : WORD);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_dpram_pointer
   ... reads the current position of the DPRAM pointer.
*)
FUNCTION m044_get_dpram_pointer (micro_slot: BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_dpram_byte
   ... writes a byte to DPRAM. The parameter 'ptr_mode' controls the
   incrementation mode of the DPRAM pointer:
   ptr_mode = _M044_FIXED_PTR : do not increase pointer
   ptr_mode = _M044_INC_PTR   : increase pointer after access
*)
PROCEDURE m044_set_dpram_byte(micro_slot, data, ptr_mode : BYTE);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_dpram_word
   ... writes a word to DPRAM. The parameter 'ptr_mode' controls the
   incrementation mode of the DPRAM pointer:
   ptr_mode = _M044_FIXED_PTR : do not increase pointer
   ptr_mode = _M044_INC_PTR   : increase pointer after access
*)
PROCEDURE m044_set_dpram_word (micro_slot : BYTE; data : WORD; ptr_mode : BYTE);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_dpram_block
   ... writes a data block to DPRAM.
*)
PROCEDURE m044_set_dpram_block (micro_slot : BYTE; VAR data; size : BYTE);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_dpram_byte
   ... reads a byte from DPRAM. The parameter 'ptr_mode' controls the
   incrementation mode of the DPRAM pointer:
   ptr_mode = _M044_FIXED_PTR : do not increase pointer
   ptr_mode = _M044_INC_PTR   : increase pointer after access
*)
FUNCTION m044_get_dpram_byte (micro_slot, ptr_mode : BYTE) : BYTE;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_dpram_word
   ... reads a word from DPRAM. The parameter 'ptr_mode' controls the
   incrementation mode of the DPRAM pointer:
   ptr_mode = _M044_FIXED_PTR : do not increase pointer
   ptr_mode = _M044_INC_PTR   : increase pointer after access
*)
FUNCTION m044_get_dpram_word(micro_slot,  ptr_mode : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_dpram_block
   ... writes a data block to DPRAM.
*)
PROCEDURE m044_get_dpram_block (micro_slot : BYTE; VAR data; size : BYTE);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_hard_reset
   ... resets FPGA and modul.
*)
FUNCTION m044_hard_reset(micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_reset_master
   ... resets master, fpga registers are not changed.
*)
FUNCTION m044_reset_master(micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_fpga_version
   ... reads the version of the FPGA of a modul M-DPM-12.
*)
FUNCTION m044_get_fpga_version(micro_slot : BYTE) : BYTE;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_modul_register
   ... sets a register of the FPGA of a modul M-DPM-12.
*)
PROCEDURE m044_set_modul_register(micro_slot, reg_type, data : BYTE);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_modul_register
        ... reads a register of the FPGA of a modul M-DPM-12.
*)
FUNCTION m044_get_modul_register(micro_slot, reg_type : BYTE) : BYTE;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_configure_io_pin
   ... configures the IO pins of the FPGA of a modul M-DPM-12.
*)
PROCEDURE m044_configure_io_pin(micro_slot : BYTE; data : WORD);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_io_pin_config
   ... reads the configuration of the IO pins
*)
FUNCTION m044_get_io_pin_config(micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_io_pin
   ... sets the IO pins of the FPGA of a modul M-DPM-12.
*)
PROCEDURE m044_set_io_pin(micro_slot : BYTE; data : WORD);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_io_pin
   ... reads the IO pins of the FPGA of a modul M-DPM-12.
*)
FUNCTION m044_get_io_pin (micro_slot : BYTE) : WORD;


(* m044_set_timeout_counter
   ... sets the timeout counter of the FPGA of a modul M-DPM-12.
*)
PROCEDURE m044_set_timeout_counter (micro_slot : BYTE; data : WORD);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_timeout_counter
   ... reads the timeout counter of the FPGA of a modul M-DPM-12.
*)
FUNCTION m044_get_timeout_counter(micro_slot : BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_cons
   ... sets a consistence request.
*)
PROCEDURE m044_set_cons(micro_slot, mode : BYTE);
{$ifdef _WIN32} stdcall; {$endif}


(* m044_clear_cons
   ... clears a consistence request and reads the FPGA status.
*)
FUNCTION m044_clear_cons(micro_slot : BYTE) : BYTE;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_modul_status
   ... reads the modul status from the FPGA.
*)
FUNCTION m044_get_modul_status(micro_slot : BYTE) : BYTE;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_get_error
        ... reads diagnosis of function call
*)
FUNCTION m044_get_error (micro_slot : BYTE; VAR data : WORD) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


(* m044_set_tasknum
        ... sets tasknum if MDD Task is used
*)
PROCEDURE m044_set_tasknum(task : WORD);
{$ifdef _WIN32} stdcall; {$endif}

(* m044_fw_timer
        ... start/stop of XTESTO Timer
*)
FUNCTION m044_fw_timer(micro_slot : BYTE;timer : WORD) : WORD;
{$ifdef _WIN32} stdcall; {$endif}


{$IFNDEF ML8RT}

{$IFDEF _WIN32}

(* m044_download_file  ... transfers a binary file to modul. *)
FUNCTION m044_download_file(micro_slot : BYTE; binary_file : str80; VAR cnt : WORD) : WORD;
(* m044_upload_file ... transfers binary datas from modul to file. *)
FUNCTION m044_upload_file(micro_slot : BYTE; binary_file : str80; VAR uploaded : WORD) : WORD;
FUNCTION Cm044_download_file(micro_slot : BYTE; binary_file : PChar; VAR cnt : WORD) : WORD; stdcall;
FUNCTION Cm044_upload_file  (micro_slot : BYTE; binary_file : PChar; VAR uploaded : WORD) : WORD; stdcall;

{$ELSE} {DOS, WIN16}

(* m044_download_file ... transfers a binary file to modul. *)
FUNCTION m044_download_file(micro_slot : BYTE; binary_file : str80; VAR cnt : WORD) : WORD;
(* m044_upload_file ... transfers binary datas from modul to file. *)
FUNCTION m044_upload_file(micro_slot : BYTE; binary_file : str80; VAR uploaded : WORD) : WORD;
FUNCTION m044_download_file_p(micro_slot : BYTE; VAR binary_file : str80; VAR cnt : WORD) : WORD;
FUNCTION m044_upload_file_p(micro_slot : BYTE; VAR binary_file : str80; VAR uploaded : WORD) : WORD;

{$ENDIF} {WIN32}

{$ENDIF} {ML8RT}



implementation



{$ifndef WINDOWS}
   {$L m044_mdd.obj}
{$endif}



FUNCTION m044_bib_startup : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_configure        (micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_master_mode (micro_slot : BYTE; VAR status : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_download (micro_slot : BYTE; VAR data; data_size, flag : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_upload (micro_slot : BYTE; VAR data; VAR data_size : BYTE; VAR flag : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_set_slave_address (micro_slot : BYTE; VAR data; size : BYTE ) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_restart(micro_slot, restart_par : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_refresh_slave_datastruct (micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_set_master_mode (micro_slot, command : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_watchdog_enable (micro_slot : BYTE; timeout : WORD) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_watchdog_retrigger (micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_set_slave_byte (micro_slot, slave, subchannel, data, cons : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_set_slave_word (micro_slot, slave, subchannel : BYTE; data : WORD; cons : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_set_slave_block (micro_slot, slave : BYTE; offset : WORD;
                               size: BYTE; VAR data; cons : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_slave_byte (micro_slot, slave, subchannel : BYTE;
                              VAR data :BYTE; cons : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_slave_word (micro_slot, slave, subchannel : BYTE;
                              VAR data : WORD; cons : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_slave_block (micro_slot, slave : BYTE; offset : WORD;
                               size : BYTE; VAR data; cons : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_set_slave_command        (micro_slot, command, group, slave : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_slave_datastruct (micro_slot, slave : BYTE;
                                    VAR data : m044_slv_datastr_type) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_slave_diagnosis_list (micro_slot : BYTE;
                                        VAR data : m044_slv_bitmap_type) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_check_slave_diagnosis (micro_slot, slave : BYTE; VAR diaglen : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_slave_diagnosis (micro_slot, slave, size : BYTE; VAR data) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_data_transfer_list(micro_slot : BYTE; VAR data : m044_slv_bitmap_type) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_check_slave_active (micro_slot, slave : BYTE; VAR active : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_master_status_struct(micro_slot : BYTE; VAR data : m044_mst_statusstr_type) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_system_error_struct(micro_slot : BYTE; VAR data : m044_mst_syserr_type) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_poll_system_error (micro_slot : BYTE; VAR syserr : WORD) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_dpram_pointer (micro_slot : BYTE; position : WORD); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_dpram_pointer (micro_slot: BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_dpram_byte(micro_slot, data, ptr_mode : BYTE); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_dpram_word (micro_slot : BYTE; data : WORD; ptr_mode : BYTE); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_dpram_block (micro_slot : BYTE; VAR data; size : BYTE); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_dpram_byte (micro_slot, ptr_mode : BYTE) : BYTE; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_dpram_word(micro_slot,  ptr_mode : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_get_dpram_block (micro_slot : BYTE; VAR data; size : BYTE); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_hard_reset(micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_reset_master(micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_fpga_version(micro_slot : BYTE) : BYTE; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_modul_register(micro_slot, reg_type, data : BYTE); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_modul_register(micro_slot, reg_type : BYTE) : BYTE; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_configure_io_pin(micro_slot : BYTE; data : WORD); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_io_pin_config(micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_io_pin(micro_slot : BYTE; data : WORD); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_io_pin (micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_timeout_counter (micro_slot : BYTE; data : WORD); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_timeout_counter(micro_slot : BYTE) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_cons(micro_slot, mode : BYTE); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_clear_cons(micro_slot : BYTE) : BYTE; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_modul_status(micro_slot : BYTE) : BYTE; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_get_error (micro_slot : BYTE; VAR data : WORD) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

PROCEDURE m044_set_tasknum(task : WORD); EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};

FUNCTION m044_fw_timer(micro_slot : BYTE;timer : WORD) : WORD; EXTERNAL
{$ifdef WINDOWS} 'm044_mdd.dll'{$ENDIF};



{$IFNDEF ML8RT}

{$IFDEF _WIN32}

FUNCTION Cm044_download_file(micro_slot : BYTE; binary_file : PChar; VAR cnt : WORD) : WORD;
         EXTERNAL 'm044_mdd.dll' NAME 'm044_download_file';
FUNCTION Cm044_upload_file(micro_slot : BYTE; binary_file : PChar; VAR uploaded : WORD) : WORD;
         EXTERNAL 'm044_mdd.dll' NAME 'm044_upload_file';

(* m044_download_file  ... transfers a binary file to modul. *)
FUNCTION m044_download_file(micro_slot : BYTE; binary_file : str80; VAR cnt : WORD) : WORD;
VAR filename : ARRAY[0..80] OF CHAR;
BEGIN
   ml_cstr(filename, binary_file, 80);
   m044_download_file := Cm044_download_file(micro_slot, filename, cnt);
END;
(* m044_upload_file ... transfers binary datas from modul to file. *)
FUNCTION m044_upload_file(micro_slot : BYTE; binary_file : str80; VAR uploaded : WORD) : WORD;
VAR filename : ARRAY[0..80] OF CHAR;
BEGIN
   ml_cstr(filename, binary_file, 80);
   m044_upload_file := Cm044_upload_file(micro_slot, filename, uploaded);
END;

{$ELSE} { DOS, WIN16}

FUNCTION m044_download_file(micro_slot : BYTE; binary_file : str80; VAR cnt : WORD) : WORD;
{$IFNDEF _WIN16}
   EXTERNAL;
{$ELSE}
BEGIN
   m044_download_file := m044_download_file_p(micro_slot, binary_file, cnt);
END;
{$ENDIF}

FUNCTION m044_upload_file(micro_slot : BYTE; binary_file : str80; VAR uploaded : WORD) : WORD;
{$IFNDEF _WIN16}
  EXTERNAL;
{$ELSE}
BEGIN
   m044_upload_file := m044_upload_file_p(micro_slot, binary_file, uploaded);
END;
{$ENDIF}


FUNCTION m044_download_file_p(micro_slot : BYTE; VAR binary_file : str80; VAR cnt : WORD) : WORD;
VAR
 handle         : FILE;
 data           : ARRAY [1..255] OF BYTE;
 error, i, bcnt : WORD;
 size, rest     : WORD;
 fsize          : LONGINT;

BEGIN
{$I-}
 Assign(handle, binary_file);
 Reset(handle, 1);
{$I+}
 IF IOResult <> 0 THEN
   BEGIN
     m044_download_file_p := $40E0;
     exit;
   END;

 fsize := FileSize(handle);
 bcnt  := Trunc(fsize / 200);
 rest  := fsize - (bcnt * 200);
 cnt := 0;
 FOR i := 1 TO bcnt DO
   BEGIN
    BlockRead(handle, data, 200, size);
      error := m044_download(micro_slot,
                             data,
                               200,
                               _M044_MORE_DATA);

      IF(error <> 0) THEN
        BEGIN
          m044_download_file_p := error;
          exit;
        END;
      cnt := cnt + 200;
    END;

 BlockRead(handle, data, rest, size);
 error := m044_download(micro_slot,
                         data,
                        rest,
                        _M044_NO_MORE_DATA);

 IF(error <> 0) THEN
   BEGIN
     m044_download_file_p := error;
     exit;
   END;

 cnt := cnt + rest;
 Close(handle);
 m044_download_file_p := 0;
{$I+}
END;


FUNCTION m044_upload_file_p
        (
         micro_slot : BYTE;            (* used slot of modul               *)
         VAR binary_file : str80;      (* binary file                      *)
         VAR uploaded : WORD           (* number of uploaded byte          *)
        ) : WORD;
VAR
 handle         : FILE;
 data           : ARRAY [1..255] OF BYTE;
 error, i, bcnt : WORD;
 flag, size     : BYTE;
 rsize          : INTEGER;

BEGIN

{$I-}
 Assign(handle, binary_file);
 Rewrite(handle, 1);

{$I+}
 IF IOResult <> 0 THEN
   BEGIN
     m044_upload_file_p := $40E0;
     exit;
   END;

 uploaded := 0;

 flag := _M044_START_DATA;

 REPEAT
   BEGIN
     error := m044_upload(micro_slot, data, size, flag);
     IF(error <> 0) THEN
       BEGIN
        m044_upload_file_p := error;
        exit;
       END;
     BlockWrite(handle, data, size, rsize);
     uploaded := uploaded + rsize;

     IF (rsize <> size) THEN
       BEGIN
        m044_upload_file_p := $40E0;
        exit;
       END;
    END;
  UNTIL ((flag = _M044_NO_MORE_DATA) OR (error <> 0));


 Close(handle);
 m044_upload_file_p := 0;
END;

{$ENDIF}
{$ENDIF}


BEGIN
END
.